/*
* Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <dlog.h>
#include "muse_camera_msg.h"
#include "muse_camera_internal.h"
#include <muse_server.h>
#include <glib.h>
#include <gst/gst.h>

#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "MUSED_CAMERA"
#define KEY_LENGTH 24

#define CAMERA_FEATURE_NAME   "http://tizen.org/feature/camera"
#define CAMERA_PRIVILEGE_NAME "http://tizen.org/privilege/camera"
#define MUSED_KEY_DEVICE_STATE_CHECK  "camera_get_device_state_is_called"
#define MUSED_KEY_DEVICE_STATE_RETURN "camera_get_device_state_return"
#define MUSED_KEY_FLASH_STATE_CHECK   "camera_get_flash_state_is_called"
#define MUSED_KEY_FLASH_STATE_RETURN  "camera_get_flash_state_return"
#define MUSED_KEY_FLASH_STATE_COUNT   "camera_get_flash_state_count"

static int _camera_remove_export_data(muse_module_h module, tbm_fd fd, int remove_all);

static void __camera_dispatcher_send_msg(muse_module_h module, char *msg, int *tfd)
{
	int len = 0;
	int sock_fd = 0;

	if (!msg) {
		LOGE("NULL msg");
		return;
	}

	if (!module) {
		LOGE("NULL module");
		goto _DONE;
	}

	/*LOGD("msg [%s]", msg);*/

	sock_fd = muse_server_module_get_msg_fd(module);
	if (sock_fd == (int)MM_ERROR_INVALID_ARGUMENT) {
		LOGE("failed to get socket fd");
		goto _DONE;
	}

	len = muse_core_msg_send_fd(sock_fd, tfd, msg);
	if (len <= 0)
		LOGE("sending message[%s] failed. errno %d", msg, errno);

_DONE:
	muse_core_msg_free(msg);

	return;
}

static void muse_camera_msg_return(int api, int class, int ret, muse_module_h module)
{
	char *send_msg = muse_core_msg_new(api,
		MUSE_TYPE_INT, PARAM_API_CLASS, class,
		MUSE_TYPE_INT, PARAM_RET, ret,
		MUSE_TYPE_INT, PARAM_GET_TYPE, MUSE_CAMERA_GET_TYPE_NONE,
		0);

	__camera_dispatcher_send_msg(module, send_msg, NULL);

	return;
}

static void muse_camera_msg_return1(int api, int class, int ret, muse_module_h module,
	int get_type, int index, const char *name, int value_int, void *value_pointer)
{
	char *send_msg = NULL;

	if (get_type == MUSE_CAMERA_GET_TYPE_INT) {
		send_msg = muse_core_msg_new(api,
			MUSE_TYPE_INT, PARAM_API_CLASS, class,
			MUSE_TYPE_INT, PARAM_RET, ret,
			MUSE_TYPE_INT, PARAM_GET_TYPE, get_type,
			MUSE_TYPE_INT, PARAM_GET_INDEX, index,
			MUSE_TYPE_INT, name, value_int,
			0);
	} else if (get_type == MUSE_CAMERA_GET_TYPE_STRING) {
		send_msg = muse_core_msg_new(api,
			MUSE_TYPE_INT, PARAM_API_CLASS, class,
			MUSE_TYPE_INT, PARAM_RET, ret,
			MUSE_TYPE_INT, PARAM_GET_TYPE, get_type,
			MUSE_TYPE_INT, PARAM_GET_INDEX, index,
			MUSE_TYPE_STRING, name, value_pointer,
			0);
	} else if (get_type == MUSE_CAMERA_GET_TYPE_POINTER) {
		send_msg = muse_core_msg_new(api,
			MUSE_TYPE_INT, PARAM_API_CLASS, class,
			MUSE_TYPE_INT, PARAM_RET, ret,
			MUSE_TYPE_INT, PARAM_GET_TYPE, get_type,
			MUSE_TYPE_INT, PARAM_GET_INDEX, index,
			MUSE_TYPE_POINTER, name, value_pointer,
			0);
	} else {
		LOGW("unknown type %d", get_type);
	}

	__camera_dispatcher_send_msg(module, send_msg, NULL);

	return;
}


static void muse_camera_msg_return2(int api, int class, int ret, muse_module_h module,
	int get_type, int index, int value0, int value1)
{
	char *send_msg = muse_core_msg_new(api,
		MUSE_TYPE_INT, PARAM_API_CLASS, class,
		MUSE_TYPE_INT, PARAM_RET, ret,
		MUSE_TYPE_INT, PARAM_GET_TYPE, get_type,
		MUSE_TYPE_INT, PARAM_GET_INDEX, index,
		MUSE_TYPE_INT, "get_value0", value0,
		MUSE_TYPE_INT, "get_value1", value1,
		0);

	__camera_dispatcher_send_msg(module, send_msg, NULL);

	return;
}


static void muse_camera_msg_event1(int api, int event, int class, muse_module_h module, const char *name, int value)
{
	char *send_msg = muse_core_msg_new(api,
		MUSE_TYPE_INT, PARAM_EVENT, event,
		MUSE_TYPE_INT, PARAM_EVENT_CLASS, class,
		MUSE_TYPE_INT, name, value,
		0);

	__camera_dispatcher_send_msg(module, send_msg, NULL);

	return;
}


static void muse_camera_msg_event2(int api, int event, int class, muse_module_h module,
	const char *name0, int value0, const char *name1, int value1)
{
	char *send_msg = muse_core_msg_new(api,
		MUSE_TYPE_INT, PARAM_EVENT, event,
		MUSE_TYPE_INT, PARAM_EVENT_CLASS, class,
		MUSE_TYPE_INT, name0, value0,
		MUSE_TYPE_INT, name1, value1,
		0);

	__camera_dispatcher_send_msg(module, send_msg, NULL);

	return;
}


static void muse_camera_msg_event2_fd(int api, int event, int class, muse_module_h module,
	const char *name0, int value0, const char *name1, int value1, int *tfd)
{
	char *send_msg = muse_core_msg_new(api,
		MUSE_TYPE_INT, PARAM_EVENT, event,
		MUSE_TYPE_INT, PARAM_EVENT_CLASS, class,
		MUSE_TYPE_INT, name0, value0,
		MUSE_TYPE_INT, name1, value1,
		0);

	__camera_dispatcher_send_msg(module, send_msg, tfd);

	return;
}


static void muse_camera_msg_event3(int api, int event, int class, muse_module_h module,
	const char *name0, int value0, const char *name1, int value1, const char *name2, int value2)
{
	char *send_msg = muse_core_msg_new(api,
		MUSE_TYPE_INT, PARAM_EVENT, event,
		MUSE_TYPE_INT, PARAM_EVENT_CLASS, class,
		MUSE_TYPE_INT, name0, value0,
		MUSE_TYPE_INT, name1, value1,
		MUSE_TYPE_INT, name2, value2,
		0);

	__camera_dispatcher_send_msg(module, send_msg, NULL);

	return;
}


static void muse_camera_msg_event3_fd(int api, int event, int class, muse_module_h module,
	const char *name0, int value0, const char *name1, int value1, const char *name2, int value2, int *tfd)
{
	char *send_msg = muse_core_msg_new(api,
		MUSE_TYPE_INT, PARAM_EVENT, event,
		MUSE_TYPE_INT, PARAM_EVENT_CLASS, class,
		MUSE_TYPE_INT, name0, value0,
		MUSE_TYPE_INT, name1, value1,
		MUSE_TYPE_INT, name2, value2,
		0);

	__camera_dispatcher_send_msg(module, send_msg, tfd);

	return;
}


bool _camera_dispatcher_callback_supported_theater_mode(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_THEATER_MODE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_af_mode(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_AF_MODE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_exposure_mode(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_EXPOSURE_MODE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_iso_mode(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_ISO,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_whitebalance(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_WHITEBALANCE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_effect(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_EFFECT,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_scene_mode(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_SCENE_MODE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_flash_mode(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_FLASH_MODE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_flash_mode2(int param1, void *user_data)
{
	int *count = (int *)user_data;

	if (count)
		(*count)++;

	return true;
}

bool _camera_dispatcher_callback_supported_fps(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_FPS,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_fps_by_resolution(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_FPS_BY_RESOLUTION,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_stream_flip(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_STREAM_FLIP,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_stream_rotation(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_STREAM_ROTATION,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_capture_format(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_CAPTURE_FORMAT,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_preview_format(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_PREVIEW_FORMAT,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

bool _camera_dispatcher_callback_supported_preview_resolution(int param1, int param2, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event2(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_PREVIEW_RESOLUTION,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1, "param2", param2);

	return true;
}

bool _camera_dispatcher_callback_supported_capture_resolution(int param1, int param2, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	muse_camera_msg_event2(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_CAPTURE_RESOLUTION,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1, "param2", param2);

	return true;
}

bool _camera_dispatcher_callback_supported_ptz_type(int param1, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	if (!module) {
		LOGE("NULL module");
		return false;
	}

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOREACH_SUPPORTED_PTZ_TYPE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module, "param1", param1);

	return true;
}

static int _camera_remove_export_data(muse_module_h module, tbm_fd fd, int remove_all)
{
	int i = 0;
	muse_camera_handle_s *muse_camera = NULL;
	GList *tmp_list = NULL;
	muse_camera_export_data *export_data = NULL;

	if (module == NULL || (fd < 0 && remove_all == FALSE)) {
		LOGE("invalid parameter %p, %d", module, fd);
		return FALSE;
	}

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);
	if (muse_camera == NULL) {
		LOGE("NULL handle");
		return FALSE;
	}

	g_mutex_lock(&muse_camera->list_lock);

	tmp_list = muse_camera->data_list;

	while (tmp_list) {
		export_data = (muse_camera_export_data *)tmp_list->data;
		tmp_list = tmp_list->next;
		if (export_data) {
			if (export_data->fd == fd || remove_all) {
				/*LOGD("fd %d matched, remove it (remove_all %d)", fd, remove_all);*/

				if (remove_all) {
					LOGW("remove remained export data fd %d, internal buffer %p",
						export_data->fd, export_data->internal_buffer);
				}

				if (export_data->is_capture) {
					LOGD("capture callback is done");
					g_cond_signal(&muse_camera->list_cond);
				}

				/* unref bo and close exported fd */
				if (export_data->bo) {
					/*LOGD("close export_data->fd %d", export_data->fd);*/

					close(export_data->fd);
					export_data->fd = -1;

					tbm_bo_unref(export_data->bo);
					export_data->bo = NULL;
				} else {
					LOGW("bo for fd %d is NULL", fd);
				}

				/* unref data_bo and exported fd */
				if (export_data->data_bo) {
					/*LOGD("close export_data->data_fd %d", export_data->data_fd);*/

					close(export_data->data_fd);
					export_data->data_fd = -1;

					tbm_bo_unref(export_data->data_bo);
					export_data->data_bo = NULL;
				}

				/* close exported fd for zero copy buffer */
				for (i = 0 ; i < export_data->num_buffer_fd ; i++) {
					/*LOGD("close export_data->buffer_fd[%d] %d", i, export_data->buffer_fd[i]);*/
					close(export_data->buffer_fd[i]);
					export_data->buffer_fd[i] = -1;
				}

				/* unref GstBuffer */
				if (export_data->internal_buffer) {
					gst_buffer_unref((GstBuffer *)export_data->internal_buffer);
					export_data->internal_buffer = NULL;
				}

				muse_camera->data_list = g_list_remove(muse_camera->data_list, export_data);

				g_free(export_data);
				export_data = NULL;

				if (remove_all == FALSE) {
					/*LOGD("fd %d, remove done", fd);*/
					g_mutex_unlock(&muse_camera->list_lock);
					return TRUE;
				} else {
					LOGD("check next data");
				}
			}
		} else {
			LOGW("NULL data");
		}
	}

	g_mutex_unlock(&muse_camera->list_lock);

	if (remove_all) {
		LOGW("remove all done");
	} else {
		LOGE("should not be reached here - fd %d", fd);
	}

	return FALSE;
}


void _camera_dispatcher_capturing_cb(camera_image_data_s* image, camera_image_data_s* postview, camera_image_data_s* thumbnail, void *user_data)
{
	muse_camera_handle_s *muse_camera = NULL;
	int tfd_index = 0;
	int data_size_main = 0;
	int data_size_post = 0;
	int data_size_thumb = 0;
	tbm_bo bo_main = NULL;
	tbm_bo bo_post = NULL;
	tbm_bo bo_thumb = NULL;
	tbm_bo_handle bo_main_handle = {.ptr = NULL};
	tbm_bo_handle bo_post_handle = {.ptr = NULL};
	tbm_bo_handle bo_thumb_handle = {.ptr = NULL};
	muse_camera_export_data *export_data_main = NULL;
	muse_camera_export_data *export_data_post = NULL;
	muse_camera_export_data *export_data_thumb = NULL;
	tbm_fd tfd[MUSE_NUM_FD] = {-1, -1, -1, -1};
	muse_module_h module = (muse_module_h)user_data;
	unsigned char *buf_pos = NULL;
	gint64 end_time = 0;

	LOGD("Enter!!");

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);
	if (!muse_camera || !image) {
		LOGE("invalid ptr %p %p", muse_camera, image);
		return;
	}

	/* main image */
	export_data_main = g_new0(muse_camera_export_data, 1);
	if (export_data_main == NULL) {
		LOGE("alloc export_data failed");
		return;
	}

	data_size_main = sizeof(camera_image_data_s) + image->size;
	if (image->exif && image->exif_size > 0)
		data_size_main += image->exif_size;

	/* alloc bo */
	bo_main = tbm_bo_alloc(muse_camera->bufmgr, data_size_main, TBM_BO_DEFAULT);
	if (bo_main == NULL) {
		LOGE("bo alloc failed");
		goto main_image_error;
	}

	bo_main_handle = tbm_bo_map(bo_main, TBM_DEVICE_CPU, TBM_OPTION_READ | TBM_OPTION_WRITE);
	if (bo_main_handle.ptr == NULL) {
		LOGE("bo map Error!");
		goto main_image_error;
	}

	buf_pos = (unsigned char *)bo_main_handle.ptr;
	memcpy(buf_pos, image, sizeof(camera_image_data_s));
	buf_pos += sizeof(camera_image_data_s);
	memcpy(buf_pos, image->data, image->size);
	if (image->exif && image->exif_size > 0) {
		buf_pos += image->size;
		memcpy(buf_pos, image->exif, image->exif_size);
	}

	tbm_bo_unmap(bo_main);

	tfd[tfd_index] = tbm_bo_export_fd(bo_main);
	if (tfd[tfd_index] < 0) {
		LOGE("export main bo ERROR");
		goto main_image_error;
	}

	LOGD("MAIN: bo %p, vaddr %p, size %d, fd %d",
		bo_main, bo_main_handle.ptr, data_size_main, tfd[tfd_index]);

	/* set bo info */
	export_data_main->fd = tfd[tfd_index];
	export_data_main->bo = bo_main;
	export_data_main->is_capture = true;

	/* postview image */
	if (postview && postview->size > 0) {
		data_size_post = sizeof(camera_image_data_s) + postview->size;

		export_data_post = g_new0(muse_camera_export_data, 1);
		if (export_data_post == NULL) {
			LOGE("alloc export_data failed");
			goto postview_image_error;
		}

		/* alloc bo */
		bo_post = tbm_bo_alloc(muse_camera->bufmgr, data_size_post, TBM_BO_DEFAULT);
		if (bo_post == NULL) {
			LOGE("bo alloc failed");
			goto postview_image_error;
		}

		bo_post_handle = tbm_bo_map(bo_post, TBM_DEVICE_CPU, TBM_OPTION_READ | TBM_OPTION_WRITE);
		if (bo_post_handle.ptr == NULL) {
			LOGE("bo map Error!");
			goto postview_image_error;
		}

		buf_pos = (unsigned char *)bo_post_handle.ptr;
		memcpy(buf_pos, postview, sizeof(camera_image_data_s));
		buf_pos += sizeof(camera_image_data_s);
		memcpy(buf_pos, postview->data, postview->size);
		tbm_bo_unmap(bo_post);

		tfd_index++;
		tfd[tfd_index] = tbm_bo_export_fd(bo_post);
		if (tfd[tfd_index] < 0) {
			LOGE("export postview ERROR");
			goto postview_image_error;
		}

		/* set bo info */
		export_data_post->fd = tfd[tfd_index];
		export_data_post->bo = bo_post;
	}

	/* thumbnail image */
	if (thumbnail && thumbnail->size > 0) {
		data_size_thumb = sizeof(camera_image_data_s) + thumbnail->size;

		export_data_thumb = g_new0(muse_camera_export_data, 1);
		if (export_data_thumb == NULL) {
			LOGE("alloc export_data failed");
			goto thumbnail_image_error;
		}

		/* alloc bo */
		bo_thumb = tbm_bo_alloc(muse_camera->bufmgr, data_size_thumb, TBM_BO_DEFAULT);
		if (bo_thumb == NULL) {
			LOGE("bo alloc failed");
			goto thumbnail_image_error;
		}

		bo_thumb_handle = tbm_bo_map(bo_thumb, TBM_DEVICE_CPU, TBM_OPTION_READ | TBM_OPTION_WRITE);
		if (bo_thumb_handle.ptr == NULL) {
			LOGE("bo map Error!");
			goto thumbnail_image_error;
		}

		buf_pos = (unsigned char *)bo_thumb_handle.ptr;
		memcpy(buf_pos, thumbnail, sizeof(camera_image_data_s));
		buf_pos += sizeof(camera_image_data_s);
		memcpy(buf_pos, thumbnail->data, thumbnail->size);
		tbm_bo_unmap(bo_thumb);

		tfd_index++;
		tfd[tfd_index] = tbm_bo_export_fd(bo_thumb);
		if (tfd[tfd_index] < 0) {
			LOGE("export thumb ERROR");
			goto thumbnail_image_error;
		}

		/* set bo info */
		export_data_thumb->fd = tfd[tfd_index];
		export_data_thumb->bo = bo_thumb;
	}

	/* add bo info to list */
	g_mutex_lock(&muse_camera->list_lock);

	muse_camera->data_list = g_list_append(muse_camera->data_list, (gpointer)export_data_main);

	if (export_data_post)
		muse_camera->data_list = g_list_append(muse_camera->data_list, (gpointer)export_data_post);

	if (export_data_thumb)
		muse_camera->data_list = g_list_append(muse_camera->data_list, (gpointer)export_data_thumb);

	/* send message */
	muse_camera_msg_event3_fd(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_CAPTURE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		module,
		"capture_fd_main", export_data_main->fd,
		"capture_fd_post", export_data_post ? export_data_post->fd : -1,
		"capture_fd_thumb", export_data_thumb ? export_data_thumb->fd : -1,
		(int *)tfd);

	/* wait for capture callback return */
	end_time = g_get_monotonic_time() + G_TIME_SPAN_SECOND * 3;
	if (!g_cond_wait_until(&muse_camera->list_cond, &muse_camera->list_lock, end_time)) {
		LOGW("capture callback return timeout");
	} else {
		LOGD("capture callback return");
	}

	g_mutex_unlock(&muse_camera->list_lock);

	return;

thumbnail_image_error:
	if (bo_thumb) {
		tbm_bo_unref(bo_thumb);
		bo_thumb = NULL;
	}

	if (export_data_thumb) {
		g_free(export_data_thumb);
		export_data_thumb = NULL;
	}

postview_image_error:
	if (bo_post) {
		tbm_bo_unref(bo_post);
		bo_post = NULL;
	}

	if (export_data_post) {
		g_free(export_data_post);
		export_data_post = NULL;
	}

main_image_error:
	if (bo_main) {
		tbm_bo_unref(bo_main);
		bo_main = NULL;
	}

	if (export_data_main) {
		g_free(export_data_main);
		export_data_main = NULL;
	}

	return;
}

void _camera_dispatcher_state_changed_cb(camera_state_e previous, camera_state_e current, bool by_policy, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;
	muse_camera_handle_s *muse_camera = NULL;
	camera_device_e device_type = CAMERA_DEVICE_CAMERA0;
	char value_key[KEY_LENGTH] = {'\0',};
	int ret = CAMERA_ERROR_NONE;
	int set_value = -1;

	LOGD("Enter - previous %d, current %d, by_policy %d",
	     previous, current, by_policy);

	muse_camera_msg_event3(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_STATE_CHANGE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_MAIN,
		module, "previous", previous, "current", current, "by_policy", by_policy);

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);
	if (!muse_camera) {
		LOGW("NULL muse camera handle");
		return;
	}

	ret = legacy_camera_get_device_type(muse_camera->camera_handle, &device_type);
	if (ret != CAMERA_ERROR_NONE) {
		LOGE("get device type failed 0x%x", ret);
		return;
	}

	snprintf(value_key, KEY_LENGTH, "device_state_camera%d", device_type);
	if (previous == CAMERA_STATE_CREATED && current == CAMERA_STATE_PREVIEW)
		set_value = CAMERA_DEVICE_STATE_WORKING;
	else if (previous == CAMERA_STATE_PREVIEW && current == CAMERA_STATE_CREATED)
		set_value = CAMERA_DEVICE_STATE_NULL;

	if (set_value != -1) {
		LOGD("device[%s] state set : %d", value_key, set_value);
		muse_server_module_set_value(module, value_key, set_value);
	}

	return;
}

void _camera_dispatcher_interrupted_cb(camera_policy_e policy, camera_state_e previous, camera_state_e current, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	LOGD("Enter - policy %d, state previous %d, current %d", policy, previous, current);

	muse_camera_msg_event3(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_INTERRUPTED,
		MUSE_CAMERA_EVENT_CLASS_THREAD_MAIN,
		module, "policy", policy, "previous", previous, "current", current);

	return;
}


void _camera_dispatcher_interrupt_started_cb(camera_policy_e policy, camera_state_e state, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	LOGD("Enter - policy %d, state %d", policy, state);

	muse_camera_msg_event2(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_INTERRUPT_STARTED,
		MUSE_CAMERA_EVENT_CLASS_THREAD_MAIN,
		module, "policy", policy, "state", state);

	return;
}

void _camera_dispatcher_preview_cb(MMCamcorderVideoStreamDataType *stream, void *user_data)
{
	int i = 0;
	int data_size = 0;
	int send_ret = 0;
	int sock_fd = 0;
	muse_camera_handle_s *muse_camera = NULL;
	tbm_bo bo = NULL;
	tbm_bo data_bo = NULL;
	tbm_bo_handle bo_handle = {.ptr = NULL};
	tbm_bo_handle data_bo_handle = {.ptr = NULL};
	/* 0: MMCamcorderVideoStreamDataType
	   1: data_bo or zero copy bo[0]
	   2: zero copy bo[1]
	   3: zero copy bo[2] */
	tbm_fd tfd[MUSE_NUM_FD] = {-1, -1, -1, -1};
	muse_camera_export_data *export_data = NULL;
	muse_module_h module = (muse_module_h)user_data;
	unsigned char *buf_pos = NULL;
	char *send_message = NULL;

	/*LOGD("Enter");*/

	if (module == NULL || stream == NULL) {
		LOGE("NULL data %p, %p", module, stream);
		return;
	}

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);
	if (muse_camera == NULL) {
		LOGE("NULL handle");
		return;
	}

	export_data = g_new0(muse_camera_export_data, 1);
	if (export_data == NULL) {
		LOGE("alloc export_data failed");
		return;
	}

	export_data->fd = -1;
	export_data->data_fd = -1;

	data_size = sizeof(MMCamcorderVideoStreamDataType);

	bo = tbm_bo_alloc(muse_camera->bufmgr, data_size, TBM_BO_DEFAULT);
	if (bo == NULL) {
		LOGE("bo alloc failed");
		goto _PREVIEW_CB_ERROR;
	}

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_CPU, TBM_OPTION_READ | TBM_OPTION_WRITE);
	if (bo_handle.ptr == NULL) {
		LOGE("bo map Error!");
		goto _PREVIEW_CB_ERROR;
	}

	buf_pos = (unsigned char *)bo_handle.ptr;

	memcpy(buf_pos, stream, sizeof(MMCamcorderVideoStreamDataType));

	tbm_bo_unmap(bo);

	if (stream->bo[0] == NULL) {
		/* non-zero copy */
		switch (stream->data_type) {
		case MM_CAM_STREAM_DATA_YUV420:
			data_size = stream->data.yuv420.length_yuv;
			break;
		case MM_CAM_STREAM_DATA_YUV422:
			data_size = stream->data.yuv422.length_yuv;
			break;
		case MM_CAM_STREAM_DATA_YUV420SP:
			data_size = stream->data.yuv420sp.length_y;
			data_size += stream->data.yuv420sp.length_uv;
			break;
		case MM_CAM_STREAM_DATA_YUV420P:
			data_size = stream->data.yuv420p.length_y;
			data_size += stream->data.yuv420p.length_u;
			data_size += stream->data.yuv420p.length_v;
			break;
		case MM_CAM_STREAM_DATA_YUV422P:
			data_size = stream->data.yuv422p.length_y;
			data_size += stream->data.yuv422p.length_u;
			data_size += stream->data.yuv422p.length_v;
			break;
		case MM_CAM_STREAM_DATA_ENCODED:
			data_size = stream->data.encoded.length_data;
			break;
		case MM_CAM_STREAM_DATA_DEPTH:
			data_size = stream->data.depth.length_data;
			break;
		case MM_CAM_STREAM_DATA_RGB:
			data_size = stream->data.rgb.length_data;
			break;
		default:
			LOGW("unknown data type %d", stream->data_type);
			break;
		}

		data_bo = tbm_bo_alloc(muse_camera->bufmgr, data_size, TBM_BO_DEFAULT);
		if (data_bo == NULL) {
			LOGE("data_bo alloc failed");
			goto _PREVIEW_CB_ERROR;
		}

		data_bo_handle = tbm_bo_map(data_bo, TBM_DEVICE_CPU, TBM_OPTION_READ | TBM_OPTION_WRITE);
		if (data_bo_handle.ptr == NULL) {
			LOGE("data_bo map Error!");
			goto _PREVIEW_CB_ERROR;
		}

		buf_pos = (unsigned char *)data_bo_handle.ptr;

		switch (stream->data_type) {
		case MM_CAM_STREAM_DATA_YUV420:
			memcpy(buf_pos, stream->data.yuv420.yuv, stream->data.yuv420.length_yuv);
			break;
		case MM_CAM_STREAM_DATA_YUV422:
			memcpy(buf_pos, stream->data.yuv422.yuv, stream->data.yuv422.length_yuv);
			break;
		case MM_CAM_STREAM_DATA_YUV420SP:
			memcpy(buf_pos, stream->data.yuv420sp.y, stream->data.yuv420sp.length_y);
			memcpy(buf_pos + stream->data.yuv420sp.length_y, stream->data.yuv420sp.uv, stream->data.yuv420sp.length_uv);
			break;
		case MM_CAM_STREAM_DATA_YUV420P:
			memcpy(buf_pos, stream->data.yuv420p.y, stream->data.yuv420p.length_y);
			memcpy(buf_pos + stream->data.yuv420p.length_y, stream->data.yuv420p.u, stream->data.yuv420p.length_u);
			memcpy(buf_pos + stream->data.yuv420p.length_y + stream->data.yuv420p.length_u, stream->data.yuv420p.v, stream->data.yuv420p.length_v);
			break;
		case MM_CAM_STREAM_DATA_YUV422P:
			memcpy(buf_pos, stream->data.yuv422p.y, stream->data.yuv422p.length_y);
			memcpy(buf_pos + stream->data.yuv422p.length_y, stream->data.yuv422p.u, stream->data.yuv422p.length_u);
			memcpy(buf_pos + stream->data.yuv422p.length_y + stream->data.yuv422p.length_u, stream->data.yuv422p.v, stream->data.yuv422p.length_v);
			break;
		case MM_CAM_STREAM_DATA_ENCODED:
			memcpy(buf_pos, stream->data.encoded.data, stream->data.encoded.length_data);
			break;
		case MM_CAM_STREAM_DATA_DEPTH:
			memcpy(buf_pos, stream->data.depth.data, stream->data.depth.length_data);
			break;
		case MM_CAM_STREAM_DATA_RGB:
			memcpy(buf_pos, stream->data.rgb.data, stream->data.rgb.length_data);
			break;
		default:
			break;
		}

		tbm_bo_unmap(data_bo);
	} else {
		/* zero copy */
		for (i = 0 ; i < MUSE_NUM_FD - 1 ; i++) {
			if (stream->bo[i]) {
				tfd[i + 1] = tbm_bo_export_fd(stream->bo[i]);
				if (tfd[i + 1] < 0) {
					LOGE("failed to export bo %p", stream->bo[i]);
					goto _PREVIEW_CB_ERROR;
				}
				export_data->buffer_fd[i] = tfd[i + 1];
				export_data->num_buffer_fd++;
			} else {
				/*LOGD("num_buffer_fd %d", export_data->num_buffer_fd);*/
				break;
			}
		}
	}

	tfd[0] = tbm_bo_export_fd(bo);
	if (tfd[0] < 0) {
		LOGE("export stream data ERROR");
		goto _PREVIEW_CB_ERROR;
	}

	/*
	LOGD("bo %p, vaddr %p, size %d, fd %d",
	     bo, bo_handle.ptr, data_size, tfd[0]);
	*/

	/* set bo info */
	export_data->fd = tfd[0];
	export_data->bo = bo;
	if (stream->internal_buffer) {
		export_data->internal_buffer = stream->internal_buffer;
		gst_buffer_ref((GstBuffer *)export_data->internal_buffer);
	}

	if (data_bo) {
		export_data->data_bo = data_bo;
		tfd[1] = tbm_bo_export_fd(data_bo);
		if (tfd[1] < 0) {
			LOGE("export data bo ERROR");
			goto _PREVIEW_CB_ERROR;
		}
		export_data->data_fd = tfd[1];
	}

	/* add bo info to list */
	g_mutex_lock(&muse_camera->list_lock);

	muse_camera->data_list = g_list_append(muse_camera->data_list, (gpointer)export_data);

	if (muse_camera->is_shutting_down) {
		LOGW("now shutting down.. skip this buffer");
		g_mutex_unlock(&muse_camera->list_lock);
		_camera_remove_export_data(module, tfd[0], FALSE);
		return;
	}

	g_mutex_unlock(&muse_camera->list_lock);

	g_mutex_lock(&muse_camera->preview_cb_lock);

	/* send message */
	send_message = muse_core_msg_new(MUSE_CAMERA_CB_EVENT,
		MUSE_TYPE_INT, PARAM_EVENT, MUSE_CAMERA_EVENT_TYPE_PREVIEW,
		MUSE_TYPE_INT, PARAM_EVENT_CLASS, MUSE_CAMERA_EVENT_CLASS_THREAD_SUB,
		MUSE_TYPE_INT, "preview_fd", tfd[0],
		MUSE_TYPE_INT, "num_buffer_fd", export_data->num_buffer_fd,
		0);

	sock_fd = muse_server_module_get_msg_fd(module);
	if (sock_fd == (int)MM_ERROR_INVALID_ARGUMENT) {
		LOGE("failed to get socket fd from module %p", module);
		send_ret = 0;
	} else {
		send_ret = muse_core_msg_send_fd(sock_fd, (int *)tfd, send_message);
	}

	muse_core_msg_free(send_message);

	/*LOGD("wait preview callback return message");*/

	if (!CHECK_PREVIEW_CB(muse_camera, PREVIEW_CB_TYPE_EVAS) && send_ret > 0) {
		gint64 end_time = g_get_monotonic_time() + G_TIME_SPAN_SECOND;

		if (!g_cond_wait_until(&muse_camera->preview_cb_cond, &muse_camera->preview_cb_lock, end_time)) {
			LOGW("preview callback return message timeout");
		} else {
			/*LOGD("preview callback return message received");*/
		}
	}

	g_mutex_unlock(&muse_camera->preview_cb_lock);

	return;

_PREVIEW_CB_ERROR:
	if (bo) {
		tbm_bo_unref(bo);
		bo = NULL;
	}

	if (data_bo) {
		tbm_bo_unref(data_bo);
		data_bo = NULL;
	}

	if (export_data) {
		if (export_data->fd >= 0) {
			close(export_data->fd);
			export_data->fd = -1;
		}

		if (export_data->data_fd >= 0) {
			close(export_data->data_fd);
			export_data->data_fd = -1;
		}

		for (i = 0 ; i < export_data->num_buffer_fd ; i++) {
			close(export_data->buffer_fd[i]);
			export_data->buffer_fd[i] = -1;
		}

		if (export_data->internal_buffer) {
			gst_buffer_unref(export_data->internal_buffer);
			export_data->internal_buffer = NULL;
		}

		g_free(export_data);
		export_data = NULL;
	}

	return;
}

void _camera_dispatcher_capture_completed_cb(void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	LOGD("Enter");

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_CAPTURE_COMPLETE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_MAIN,
		module, "none", 0);

	return;
}

void _camera_dispatcher_face_detected_cb(camera_detected_face_s *faces, int count, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;
	muse_camera_handle_s *muse_camera = NULL;
	tbm_bo bo = NULL;
	tbm_bo_handle bo_handle = {NULL, };
	int bo_size = sizeof(camera_detected_face_s) * count;
	tbm_fd tfd[MUSE_NUM_FD] = {-1, -1, -1, -1};
	muse_camera_export_data *export_data = NULL;

	if (count >= 0) {
		if (module == NULL) {
			LOGE("NULL module");
			return;
		}

		if (bo_size > 0) {
			muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);
			if (muse_camera == NULL) {
				LOGE("NULL handle");
				return;
			}

			export_data = g_new0(muse_camera_export_data, 1);
			if (export_data == NULL) {
				LOGE("alloc export_data failed");
				return;
			}

			bo = tbm_bo_alloc(muse_camera->bufmgr, bo_size, TBM_BO_DEFAULT);
			if (bo == NULL) {
				LOGE("tbm_bo_alloc failed");

				g_free(export_data);
				export_data = NULL;

				return;
			}

			bo_handle = tbm_bo_map(bo, TBM_DEVICE_CPU, TBM_OPTION_READ | TBM_OPTION_WRITE);
			if (bo_handle.ptr == NULL) {
				LOGE("bo map Error!");

				tbm_bo_unref(bo);
				bo = NULL;

				g_free(export_data);
				export_data = NULL;

				return;
			}

			/* copy face detection info */
			memcpy(bo_handle.ptr, faces, bo_size);

			tbm_bo_unmap(bo);

			/* export bo */
			tfd[0] = tbm_bo_export_fd(bo);
			if (tfd[0] < 0) {
				LOGE("failed to export bo for face detection info");

				tbm_bo_unref(bo);
				bo = NULL;

				g_free(export_data);
				export_data = NULL;

				return;
			}

			/* set export data */
			export_data->bo = bo;
			export_data->fd = tfd[0];

			/* add bo info to list */
			g_mutex_lock(&muse_camera->list_lock);
			muse_camera->data_list = g_list_append(muse_camera->data_list, (gpointer)export_data);
			g_mutex_unlock(&muse_camera->list_lock);
		}

		LOGD("face - count %d, buffer size %d, fd %d", count, bo_size, tfd[0]);

		/* send message */
		muse_camera_msg_event2_fd(MUSE_CAMERA_CB_EVENT,
			MUSE_CAMERA_EVENT_TYPE_FACE_DETECTION,
			MUSE_CAMERA_EVENT_CLASS_THREAD_MAIN,
			module,
			"count", count,
			"face_fd", tfd[0],
			(int *)tfd);
	} else {
		LOGW("invalid count for face detection - %d", count);
	}

	return;
}

void _camera_dispatcher_focus_changed_cb(camera_focus_state_e state, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	LOGD("Enter - state %d", state);

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_FOCUS_CHANGE,
		MUSE_CAMERA_EVENT_CLASS_THREAD_MAIN,
		module, "state", state);

	return;
}

void _camera_dispatcher_error_cb(camera_error_e error, camera_state_e current_state, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	LOGD("Enter - error 0x%x, current_state %d", error, current_state);

	muse_camera_msg_event2(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_ERROR,
		MUSE_CAMERA_EVENT_CLASS_THREAD_MAIN,
		module, "error", error, "current_state", current_state);

	return;
}

void _camera_dispatcher_hdr_progress_cb(int percent, void *user_data)
{
	muse_module_h module = (muse_module_h)user_data;

	LOGD("Enter");

	muse_camera_msg_event1(MUSE_CAMERA_CB_EVENT,
		MUSE_CAMERA_EVENT_TYPE_HDR_PROGRESS,
		MUSE_CAMERA_EVENT_CLASS_THREAD_MAIN,
		module, "percent", percent);

	return;
}


static void _camera_task_add_job(muse_camera_handle_s *muse_camera, int api, int class, int value)
{
	muse_camera_task_job_s *job = NULL;

	if (!muse_camera) {
		LOGE("NULL handle");
		return;
	}

	job = g_new0(muse_camera_task_job_s, 1);
	if (!job) {
		LOGE("job alloc failed");
		muse_camera_msg_return(api, class, CAMERA_ERROR_INVALID_OPERATION, muse_camera->module);
		return;
	}

	job->api = api;
	job->class = class;
	job->value = value;

	LOGD("push job - api %d", api);

	g_mutex_lock(&muse_camera->task_lock);
	g_queue_push_tail(&muse_camera->task_queue, (gpointer)job);
	g_cond_signal(&muse_camera->task_cond);
	g_mutex_unlock(&muse_camera->task_lock);

	return;
}


static void __camera_task_process_job(muse_camera_handle_s *muse_camera, muse_camera_task_job_s *job)
{
	int i = 0;
	int ret = CAMERA_ERROR_NONE;

	if (!muse_camera) {
		LOGE("NULL handle");
		goto _PROCESS_DONE;
	}

	LOGD("job start - api %d, value 0x%x", job->api, job->value);

	switch (job->api) {
	case MUSE_CAMERA_API_STOP_PREVIEW:
		ret = legacy_camera_stop_preview(muse_camera->camera_handle);

		if (ret == CAMERA_ERROR_NONE && muse_camera->fd_number > 0) {
			for (i = 0 ; i < muse_camera->fd_number ; i++) {
				LOGD("close user buffer fd[%d] %d", i, muse_camera->fds[i]);
				close(muse_camera->fds[i]);
				muse_camera->fds[i] = -1;
			}
			muse_camera->fd_number = 0;
		}
		break;
	case MUSE_CAMERA_API_START_CAPTURE:
		ret = legacy_camera_start_capture(muse_camera->camera_handle,
			(camera_capturing_cb)_camera_dispatcher_capturing_cb,
			(camera_capture_completed_cb)_camera_dispatcher_capture_completed_cb,
			(void *)muse_camera->module);
		break;
	case MUSE_CAMERA_API_SET_PREVIEW_RESOLUTION:
		ret = legacy_camera_set_preview_resolution(muse_camera->camera_handle, \
			job->value >> 16, 0x0000ffff & job->value);
		break;
	case MUSE_CAMERA_API_SET_CAPTURE_RESOLUTION:
		ret = legacy_camera_set_capture_resolution(muse_camera->camera_handle, \
			job->value >> 16, 0x0000ffff & job->value);
		break;
	case MUSE_CAMERA_API_ATTR_SET_HDR_MODE:
		ret = legacy_camera_attr_set_hdr_mode(muse_camera->camera_handle, \
			(camera_attr_hdr_mode_e)job->value);
		break;
	case MUSE_CAMERA_API_SET_PREVIEW_CB:
		ret = legacy_camera_set_preview_cb(muse_camera->camera_handle, \
			(camera_preview_cb)_camera_dispatcher_preview_cb,
			(void *)muse_camera->module);

		if (ret == CAMERA_ERROR_NONE)
			SET_PREVIEW_CB_TYPE(muse_camera, PREVIEW_CB_TYPE_USER);
		break;
	case MUSE_CAMERA_API_UNSET_PREVIEW_CB:
		UNSET_PREVIEW_CB_TYPE(muse_camera, PREVIEW_CB_TYPE_USER);
		if (CHECK_PREVIEW_CB(muse_camera, PREVIEW_CB_TYPE_EVAS))
			LOGW("Preview callback for evas surface is remained.");
		else
			ret = legacy_camera_unset_preview_cb(muse_camera->camera_handle);
		break;
	case MUSE_CAMERA_API_SET_MEDIA_PACKET_PREVIEW_CB:
		ret = legacy_camera_set_media_packet_preview_cb(muse_camera->camera_handle,
			(camera_preview_cb)_camera_dispatcher_preview_cb,
			(void *)muse_camera->module);
		break;
	case MUSE_CAMERA_API_UNSET_MEDIA_PACKET_PREVIEW_CB:
		ret = legacy_camera_unset_media_packet_preview_cb(muse_camera->camera_handle);
		break;
	default:
		LOGE("unhandled task - api %d", job->api);
		goto _PROCESS_DONE;
	}

	LOGD("job done - api %d, ret 0x%x", job->api, ret);

	muse_camera_msg_return(job->api, job->class, ret, muse_camera->module);

_PROCESS_DONE:
	g_free(job);
	job = NULL;

	return;
}


static void *_camera_dispatcher_task_func(gpointer data)
{
	muse_camera_handle_s *muse_camera = (muse_camera_handle_s *)data;
	muse_camera_task_job_s *job = NULL;
	gint64 end_time = 0;
	bool is_signaled = false;
	bool use_wait_until = false;

	if (!muse_camera) {
		LOGE("NULL handle");
		return NULL;
	}

	LOGW("enter");

	g_mutex_lock(&muse_camera->task_lock);

	while (muse_camera->task_run) {
		if (g_queue_is_empty(&muse_camera->task_queue)) {
			LOGD("empty queue. wait signal [wait until %d]", use_wait_until);

			if (use_wait_until) {
				end_time = g_get_monotonic_time() + G_TIME_SPAN_SECOND;
				is_signaled = g_cond_wait_until(&muse_camera->task_cond, &muse_camera->task_lock, end_time);
			} else {
				g_cond_wait(&muse_camera->task_cond, &muse_camera->task_lock);
				is_signaled = true;
			}

			/*LOGD("is_signaled %d", is_signaled);*/
		}

		if (!muse_camera->task_run) {
			LOGW("stop task thread : is_signaled %d", is_signaled);
			break;
		}

		job = (muse_camera_task_job_s *)g_queue_pop_head(&muse_camera->task_queue);
		if (!job) {
			if (is_signaled)
				LOGE("signal received, but no job");

			continue;
		}

		if (job->api == MUSE_CAMERA_API_STOP_PREVIEW)
			use_wait_until = true;
		else
			use_wait_until = false;

		g_mutex_unlock(&muse_camera->task_lock);

		__camera_task_process_job(muse_camera, job);

		g_mutex_lock(&muse_camera->task_lock);
	}

	while (!g_queue_is_empty(&muse_camera->task_queue)) {
		job = (muse_camera_task_job_s *)g_queue_pop_head(&muse_camera->task_queue);
		if (job) {
			LOGW("remained job - api %d", job->api);
			__camera_task_process_job(muse_camera, job);
		}
	}

	g_mutex_unlock(&muse_camera->task_lock);

	LOGW("leave");

	return NULL;
}


static void _camera_dispatcher_release_resource(muse_module_h module)
{
	muse_camera_handle_s *muse_camera = NULL;

	if (!module) {
		LOGE("NULL handle");
		return;
	}

	LOGW("enter");

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	_camera_remove_export_data(module, -1, TRUE);

	g_mutex_clear(&muse_camera->list_lock);
	g_cond_clear(&muse_camera->list_cond);
	g_mutex_clear(&muse_camera->preview_cb_lock);
	g_cond_clear(&muse_camera->preview_cb_cond);

	muse_camera->bufmgr = NULL;

	if (muse_camera->task_thread) {
		LOGW("task thread exiting start");

		g_mutex_lock(&muse_camera->task_lock);
		muse_camera->task_run = false;
		g_cond_signal(&muse_camera->task_cond);
		g_mutex_unlock(&muse_camera->task_lock);

		LOGW("wait task thread join");

		g_thread_join(muse_camera->task_thread);
		muse_camera->task_thread = NULL;

		LOGW("task thread exiting end");
	}

	g_mutex_clear(&muse_camera->task_lock);
	g_cond_clear(&muse_camera->task_cond);
	g_queue_clear(&muse_camera->task_queue);

	free(muse_camera);
	muse_camera = NULL;

	LOGW("leave");

	return;
}


int camera_dispatcher_create(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	int device_type = 0;
	int preview_format = CAMERA_PIXEL_FORMAT_INVALID;
	int user_buffer_supported = 0;
	int pid = 0;
	bool camera_feature_supported = false;
	char *send_msg = NULL;
	void *gdbus_connection = NULL;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_CREATE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera_msg_get(device_type, muse_server_module_get_msg(module));
	muse_camera_msg_get(pid, muse_server_module_get_msg(module));

	LOGD("device type : %d, client pid : %d", device_type, pid);

	/* init handle */
	muse_camera = (muse_camera_handle_s *)malloc(sizeof(muse_camera_handle_s));
	if (muse_camera == NULL) {
		ret = CAMERA_ERROR_OUT_OF_MEMORY;
		LOGE("handle alloc failed 0x%x", ret);
		muse_camera_msg_return(api, class, ret, module);
		return MUSE_CAMERA_ERROR_INVALID;
	}

	memset(muse_camera, 0x0, sizeof(muse_camera_handle_s));

	if (muse_server_ipc_get_bufmgr(&muse_camera->bufmgr) != MM_ERROR_NONE ||
		muse_server_ipc_get_gdbus_connection((GDBusConnection **)&gdbus_connection) != MM_ERROR_NONE) {
		LOGE("tbm bufmgr or gdbus conntection failed %p %p", muse_camera->bufmgr, gdbus_connection);

		free(muse_camera);
		muse_camera = NULL;

		ret = CAMERA_ERROR_INVALID_OPERATION;
		muse_camera_msg_return(api, class, ret, module);

		return MUSE_CAMERA_ERROR_INVALID;
	}

	ret = legacy_camera_create((camera_device_e)device_type, &muse_camera->camera_handle);
	if (ret != CAMERA_ERROR_NONE) {
		free(muse_camera);
		muse_camera = NULL;
		muse_camera_msg_return(api, class, ret, module);

		return MUSE_CAMERA_ERROR_INVALID;
	}

	g_mutex_init(&muse_camera->task_lock);
	g_cond_init(&muse_camera->task_cond);
	g_queue_init(&muse_camera->task_queue);

	/* camera feature support check */
	if (muse_server_get_platform_info(CAMERA_FEATURE_NAME, &camera_feature_supported) != MM_ERROR_NONE)
		LOGE("failed to get platform info for camera support");

	if (!camera_feature_supported) {
		LOGE("CAMERA NOT SUPPORTED, BUT INI FILE FOR CAMERA IS EXISTED. RECOMMEND TO REMOVE IT.");
		ret = CAMERA_ERROR_NOT_SUPPORTED;
		goto _CREATE_ERROR;
	}

	muse_camera->task_run = true;
	muse_camera->task_thread = g_thread_try_new("camera_task_thread",
		_camera_dispatcher_task_func, (gpointer)muse_camera, NULL);
	if (!muse_camera->task_thread) {
		LOGE("failed to create new thread for task");
		goto _CREATE_ERROR;
	}

	ret = legacy_camera_set_state_changed_cb(muse_camera->camera_handle,
		(camera_state_changed_cb)_camera_dispatcher_state_changed_cb,
		(void *)module);
	if (ret != CAMERA_ERROR_NONE) {
		LOGE("legacy_camera_set_state_changed_cb failed : 0x%x", ret);
		goto _CREATE_ERROR;
	}

	ret = legacy_camera_set_client_pid(muse_camera->camera_handle, pid);
	if (ret != CAMERA_ERROR_NONE) {
		LOGE("legacy_camera_set_client_pid failed : 0x%x", ret);
		goto _CREATE_ERROR;
	}

	ret = legacy_camera_set_gdbus_connection(muse_camera->camera_handle, gdbus_connection);
	if (ret != CAMERA_ERROR_NONE) {
		LOGE("legacy_camera_set_gdbus_connection failed : 0x%x", ret);
		goto _CREATE_ERROR;
	}

	ret = legacy_camera_get_preview_format(muse_camera->camera_handle, &preview_format);
	if (ret != CAMERA_ERROR_NONE) {
		LOGE("legacy_camera_get_preview_format failed : 0x%x", ret);
		goto _CREATE_ERROR;
	}

	user_buffer_supported = legacy_camera_attr_is_supported_user_buffer(muse_camera->camera_handle);

	g_mutex_init(&muse_camera->list_lock);
	g_cond_init(&muse_camera->list_cond);
	g_mutex_init(&muse_camera->preview_cb_lock);
	g_cond_init(&muse_camera->preview_cb_cond);
	muse_camera->preview_cb_flag = 0;
	muse_camera->module = module;

	LOGD("handle : %p, user buffer %d", muse_camera, user_buffer_supported);

	muse_server_ipc_set_handle(module, (intptr_t)muse_camera);

	send_msg = muse_core_msg_new(api,
		MUSE_TYPE_INT, PARAM_API_CLASS, class,
		MUSE_TYPE_INT, PARAM_RET, ret,
		MUSE_TYPE_INT, "preview_format", preview_format,
		MUSE_TYPE_INT, "user_buffer_supported", user_buffer_supported,
		MUSE_TYPE_POINTER, "handle", muse_camera,
		0);

	__camera_dispatcher_send_msg(module, send_msg, NULL);

	return MUSE_CAMERA_ERROR_NONE;

_CREATE_ERROR:
	if (muse_camera->task_thread) {
		g_mutex_lock(&muse_camera->task_lock);
		muse_camera->task_run = false;
		g_cond_signal(&muse_camera->task_cond);
		g_mutex_unlock(&muse_camera->task_lock);

		LOGE("task thread join");

		g_thread_join(muse_camera->task_thread);
		muse_camera->task_thread = NULL;
	}

	g_mutex_clear(&muse_camera->task_lock);
	g_cond_clear(&muse_camera->task_cond);
	g_queue_clear(&muse_camera->task_queue);

	legacy_camera_destroy(muse_camera->camera_handle);
	muse_camera->camera_handle = NULL;

	free(muse_camera);
	muse_camera = NULL;
	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_INVALID;
}


int camera_dispatcher_change_device(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	int device = CAMERA_DEVICE_CAMERA0;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_CHANGE_DEVICE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(device, muse_server_module_get_msg(module));

	LOGD("change device to %d", device);

	ret = legacy_camera_change_device(&muse_camera->camera_handle, device,
		CHECK_PREVIEW_CB(muse_camera, PREVIEW_CB_TYPE_EVAS));

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}


int camera_dispatcher_destroy(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_DESTROY;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("Enter, handle : %p", muse_camera);

	ret = legacy_camera_destroy(muse_camera->camera_handle);
	if (ret == CAMERA_ERROR_NONE)
		_camera_dispatcher_release_resource(module);

	muse_camera_msg_return(api, class, ret, module);

	if (ret == CAMERA_ERROR_NONE)
		return MUSE_CAMERA_ERROR_NONE;
	else
		return MUSE_CAMERA_ERROR_INVALID;
}

int camera_dispatcher_start_preview(muse_module_h module)
{
	int i = 0;
	int ret = CAMERA_ERROR_NONE;
	int client_fd = -1;
	int fds[MUSE_NUM_FD] = {-1, -1, -1, -1};
	int fd_number = 0;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_START_PREVIEW;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	/* privilege check */
	client_fd = muse_server_module_get_msg_fd(module);
	if (client_fd < 0) {
		ret = CAMERA_ERROR_INVALID_OPERATION;
		LOGE("failed to get fd %d", client_fd);
		muse_camera_msg_return(api, class, ret, module);
		return MUSE_CAMERA_ERROR_INVALID;
	}

	if (!muse_server_security_check_cynara(client_fd, CAMERA_PRIVILEGE_NAME)) {
		ret = CAMERA_ERROR_PERMISSION_DENIED;
		LOGE("security check failed 0x%x", ret);
		muse_camera_msg_return(api, class, ret, module);
		return MUSE_CAMERA_ERROR_INVALID;
	}

	/* get user buffer fd */
	if (muse_server_ipc_get_fds(module, fds, &fd_number) == MM_ERROR_NONE) {
		/* set user buffer */
		if (fd_number > 0) {
			ret = legacy_camera_set_user_buffer(muse_camera->camera_handle, fds, fd_number);
			LOGD("fd number %d, ret 0x%x", fd_number, ret);
		}
	}

	ret |= legacy_camera_start_preview(muse_camera->camera_handle);

	muse_camera_msg_return(api, class, ret, module);

	/* fd handling */
	if (fd_number > 0) {
		if (ret == CAMERA_ERROR_NONE) {
			/* set fd and number */
			for (i = 0 ; i < fd_number ; i++)
				muse_camera->fds[i] = fds[i];

			muse_camera->fd_number = fd_number;
		} else {
			/* close exported fd */
			for (i = 0 ; i < fd_number ; i++) {
				LOGE("[PREVIEW FAILED] close exported fds[%d] %d", i, fds[i]);
				close(fds[i]);
			}

			muse_camera->fd_number = 0;
		}
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_stop_preview(muse_module_h module)
{
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	_camera_task_add_job(muse_camera, MUSE_CAMERA_API_STOP_PREVIEW,
		MUSE_CAMERA_API_CLASS_IMMEDIATE, 0);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_start_capture(muse_module_h module)
{
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	_camera_task_add_job(muse_camera, MUSE_CAMERA_API_START_CAPTURE,
		MUSE_CAMERA_API_CLASS_IMMEDIATE, 0);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_is_supported_continuous_capture(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SUPPORT_CONTINUOUS_CAPTURE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_is_supported_continuous_capture(muse_camera->camera_handle);

	LOGD("is supported ret : %d", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_start_continuous_capture(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int value = 0;
	int count = 0;
	int interval = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_START_CONTINUOUS_CAPTURE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(value, muse_server_module_get_msg(module));

	count = value >> 16;
	interval = 0x0000ffff & value;

	LOGD("Enter - count %d, interval %d", count, interval);

	ret = legacy_camera_start_continuous_capture(muse_camera->camera_handle,
		count,
		interval,
		(camera_capturing_cb)_camera_dispatcher_capturing_cb,
		(camera_capture_completed_cb)_camera_dispatcher_capture_completed_cb,
		(void *)module);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_stop_continuous_capture(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_STOP_CONTINUOUS_CAPTURE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_stop_continuous_capture(muse_camera->camera_handle);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_is_supported_face_detection(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SUPPORT_FACE_DETECTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_is_supported_face_detection(muse_camera->camera_handle);

	LOGD("is supported ret : %d", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_is_supported_zero_shutter_lag(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SUPPORT_ZERO_SHUTTER_LAG;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_is_supported_zero_shutter_lag(muse_camera->camera_handle);

	LOGD("is supported ret : %d", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_is_supported_media_packet_preview_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SUPPORT_MEDIA_PACKET_PREVIEW_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_is_supported_media_packet_preview_cb(muse_camera->camera_handle);

	LOGD("is supported ret : %d", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_device_count(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_DEVICE_COUNT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_device_count(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_DEVICE_COUNT, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_start_face_detection(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_START_FACE_DETECTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("Enter, handle : %p, module : %p", muse_camera, module);

	ret = legacy_camera_start_face_detection(muse_camera->camera_handle,
		(camera_face_detected_cb)_camera_dispatcher_face_detected_cb,
		(void *)module);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_stop_face_detection(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_STOP_FACE_DETECTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_stop_face_detection(muse_camera->camera_handle);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_state(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_state_e get_value = CAMERA_STATE_NONE;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_STATE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_state(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_STATE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_start_focusing(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int is_continuous;
	muse_camera_api_e api = MUSE_CAMERA_API_START_FOCUSING;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(is_continuous, muse_server_module_get_msg(module));

	LOGD("Enter, handle : %p, module : %p", muse_camera, module);

	ret = legacy_camera_start_focusing(muse_camera->camera_handle, (bool)is_continuous);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_stop_focusing(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_CANCEL_FOCUSING;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_cancel_focusing(muse_camera->camera_handle);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_display(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_DISPLAY;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	muse_camera_display_info_s dp_info = {0, };
	camera_h camera = NULL;;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	camera = muse_camera->camera_handle;

	muse_core_msg_deserialize("dp_info", muse_server_module_get_msg(module),
		NULL, NULL, MUSE_TYPE_ARRAY, (void *)&dp_info);

	LOGD("type %d, parent id %d (%d,%d,%dx%d)",
		dp_info.type, dp_info.parent_id,
		dp_info.window_rect.x, dp_info.window_rect.y,
		dp_info.window_rect.width, dp_info.window_rect.height);

	/* set window info */
	muse_camera->window_info.surface_id = dp_info.parent_id;
	muse_camera->window_info.rect = dp_info.window_rect;

	switch (dp_info.type) {
	case CAMERA_DISPLAY_TYPE_OVERLAY:
		LOGD("wayland global surface id : %d", muse_camera->window_info.surface_id);
		ret = legacy_camera_set_display(muse_camera->camera_handle,
			CAMERA_DISPLAY_TYPE_OVERLAY, (void *)&muse_camera->window_info);
		break;
	case CAMERA_DISPLAY_TYPE_EVAS:
		ret = legacy_camera_set_preview_cb(muse_camera->camera_handle,
			(camera_preview_cb)_camera_dispatcher_preview_cb, (void *)module);
		if (ret == CAMERA_ERROR_NONE)
			SET_PREVIEW_CB_TYPE(muse_camera, PREVIEW_CB_TYPE_EVAS);
		/* fall through */
	default:
		LOGD("NOT overlay type. set NONE type.");
		ret = legacy_camera_set_display(muse_camera->camera_handle,
			CAMERA_DISPLAY_TYPE_NONE, NULL);
		break;
	}

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_preview_resolution(muse_module_h module)
{
	int value = 0;
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(value, muse_server_module_get_msg(module));

	_camera_task_add_job(muse_camera, MUSE_CAMERA_API_SET_PREVIEW_RESOLUTION,
		MUSE_CAMERA_API_CLASS_IMMEDIATE, value);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_capture_resolution(muse_module_h module)
{
	int value = 0;
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(value, muse_server_module_get_msg(module));

	_camera_task_add_job(muse_camera, MUSE_CAMERA_API_SET_CAPTURE_RESOLUTION,
		MUSE_CAMERA_API_CLASS_IMMEDIATE, value);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_capture_format(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_format;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_CAPTURE_FORMAT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_format, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_format : %d", muse_camera, set_format);

	ret = legacy_camera_set_capture_format(muse_camera->camera_handle, (camera_pixel_format_e)set_format);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_preview_format(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_format;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_PREVIEW_FORMAT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_format, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_format : %d", muse_camera, set_format);

	ret = legacy_camera_set_preview_format(muse_camera->camera_handle, (camera_pixel_format_e)set_format);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_preview_resolution(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_width = 0;
	int get_height = 0;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_PREVIEW_RESOLUTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_preview_resolution(muse_camera->camera_handle, &get_width, &get_height);
	if (ret == CAMERA_ERROR_NONE) {
		get_value = get_width << 16 | get_height;
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_PREVIEW_RESOLUTION, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_display_rotation(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_rotation;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_DISPLAY_ROTATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_rotation, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_rotation : %d", muse_camera, set_rotation);

	ret = legacy_camera_set_display_rotation(muse_camera->camera_handle, (camera_rotation_e)set_rotation);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_display_rotation(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_rotation_e get_value = CAMERA_ROTATION_NONE;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_DISPLAY_ROTATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_display_rotation(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_DISPLAY_ROTATION, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_display_flip(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_flip;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_DISPLAY_FLIP;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_flip, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_flip : %d", muse_camera, set_flip);

	ret = legacy_camera_set_display_flip(muse_camera->camera_handle, (camera_flip_e)set_flip);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_display_flip(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_flip_e get_value = CAMERA_FLIP_NONE;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_DISPLAY_FLIP;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_display_flip(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_DISPLAY_FLIP, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_display_visible(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_visible;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_DISPLAY_VISIBLE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_visible, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_visible : %d", muse_camera, set_visible);

	ret = legacy_camera_set_display_visible(muse_camera->camera_handle, (bool)set_visible);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_is_display_visible(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	bool get_value = true;
	muse_camera_api_e api = MUSE_CAMERA_API_IS_DISPLAY_VISIBLE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_is_display_visible(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_DISPLAY_VISIBLE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_display_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_mode;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_DISPLAY_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_mode, muse_server_module_get_msg(module));

	LOGD("handle : %p, display_mode : %d", muse_camera, set_mode);

	ret = legacy_camera_set_display_mode(muse_camera->camera_handle, (camera_display_mode_e)set_mode);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_display_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_display_mode_e get_value = CAMERA_DISPLAY_MODE_LETTER_BOX;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_DISPLAY_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_display_mode(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_DISPLAY_MODE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_capture_resolution(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_width = 0;
	int get_height = 0;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_CAPTURE_RESOLUTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_capture_resolution(muse_camera->camera_handle, &get_width, &get_height);
	if (ret == CAMERA_ERROR_NONE) {
		get_value = get_width << 16 | get_height;
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_CAPTURE_RESOLUTION, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_capture_format(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_pixel_format_e get_value = CAMERA_PIXEL_FORMAT_NV12;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_CAPTURE_FORMAT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_capture_format(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_CAPTURE_FORMAT, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_preview_format(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_pixel_format_e get_value = CAMERA_PIXEL_FORMAT_NV12;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_PREVIEW_FORMAT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_preview_format(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_PREVIEW_FORMAT, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_facing_direction(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_facing_direction_e get_value = CAMERA_FACING_DIRECTION_REAR;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_FACING_DIRECTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_facing_direction(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_FACING_DIRECTION, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_flash_state(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	int count = 0;
	int is_called = false;
	char value_key[KEY_LENGTH] = {'\0',};
	camera_h camera = NULL;
	camera_device_e device_type = CAMERA_DEVICE_CAMERA0;
	camera_flash_state_e get_flash_state = CAMERA_FLASH_STATE_NOT_USED;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_FLASH_STATE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera_msg_get(device_type, muse_server_module_get_msg(module));

	if (device_type > CAMERA_DEVICE_CAMERA9) {
		LOGE("invalid device %d", device_type);

		ret = CAMERA_ERROR_INVALID_PARAMETER;
		muse_camera_msg_return(api, class, ret, module);

		return MUSE_CAMERA_ERROR_NONE;
	}

	muse_server_module_get_value(module, MUSED_KEY_FLASH_STATE_CHECK, &is_called);

	if (!is_called) {
		ret = legacy_camera_create(device_type, &camera);
		if (camera) {
			legacy_camera_attr_foreach_supported_flash_mode(camera,
				(camera_attr_supported_flash_mode_cb)_camera_dispatcher_callback_supported_flash_mode2,
				(void *)&count);

			legacy_camera_destroy(camera);
			camera = NULL;
		}

		muse_server_module_set_value(module, MUSED_KEY_FLASH_STATE_RETURN, ret);
		muse_server_module_set_value(module, MUSED_KEY_FLASH_STATE_COUNT, count);
		muse_server_module_set_value(module, MUSED_KEY_FLASH_STATE_CHECK, (int)true);
	} else {
		muse_server_module_get_value(module, MUSED_KEY_FLASH_STATE_RETURN, &ret);
		muse_server_module_get_value(module, MUSED_KEY_FLASH_STATE_COUNT, &count);
	}

	LOGD("is_called %d, ret 0x%x, count %d", is_called, ret, count);

	if (ret != CAMERA_ERROR_NONE) {
		LOGE("failed to create or get camera info 0x%x", ret);

		muse_camera_msg_return(api, class, ret, module);

		return MUSE_CAMERA_ERROR_NONE;
	}

	if (count < 2) {
		LOGE("count[%d] of supported flash mode is too small, so return NOT_SUPPORTED", count);

		ret = CAMERA_ERROR_NOT_SUPPORTED;
		muse_camera_msg_return(api, class, ret, module);

		return MUSE_CAMERA_ERROR_NONE;
	}

	snprintf(value_key, KEY_LENGTH, "flash_state_camera%d", device_type);
	muse_server_module_get_value(module, value_key, (int *)&get_flash_state);

	LOGD("[%d] flash state : %d", device_type, get_flash_state);

	muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			-1, "get_flash_state", get_flash_state, NULL);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_preview_cb(muse_module_h module)
{
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	_camera_task_add_job(muse_camera, MUSE_CAMERA_API_SET_PREVIEW_CB,
		MUSE_CAMERA_API_CLASS_IMMEDIATE, 0);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_unset_preview_cb(muse_module_h module)
{
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	_camera_task_add_job(muse_camera, MUSE_CAMERA_API_UNSET_PREVIEW_CB,
		MUSE_CAMERA_API_CLASS_IMMEDIATE, 0);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_media_packet_preview_cb(muse_module_h module)
{
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	_camera_task_add_job(muse_camera, MUSE_CAMERA_API_SET_MEDIA_PACKET_PREVIEW_CB,
		MUSE_CAMERA_API_CLASS_IMMEDIATE, 0);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_unset_media_packet_preview_cb(muse_module_h module)
{
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	_camera_task_add_job(muse_camera, MUSE_CAMERA_API_UNSET_MEDIA_PACKET_PREVIEW_CB,
		MUSE_CAMERA_API_CLASS_IMMEDIATE, 0);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_state_changed_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_STATE_CHANGED_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	LOGD("Enter");

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_unset_state_changed_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_api_e api = MUSE_CAMERA_API_UNSET_STATE_CHANGED_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	LOGD("Enter");

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_interrupted_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_INTERRUPTED_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_set_interrupted_cb(muse_camera->camera_handle,
		(camera_interrupted_cb)_camera_dispatcher_interrupted_cb,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}


int camera_dispatcher_unset_interrupted_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_UNSET_INTERRUPTED_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_unset_interrupted_cb(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}


int camera_dispatcher_set_interrupt_started_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_INTERRUPT_STARTED_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_set_interrupt_started_cb(muse_camera->camera_handle,
		(camera_interrupt_started_cb)_camera_dispatcher_interrupt_started_cb,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}


int camera_dispatcher_unset_interrupt_started_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_UNSET_INTERRUPT_STARTED_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_unset_interrupt_started_cb(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_focus_changed_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_FOCUS_CHANGED_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_set_focus_changed_cb(muse_camera->camera_handle,
		(camera_focus_changed_cb)_camera_dispatcher_focus_changed_cb,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_unset_focus_changed_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_UNSET_FOCUS_CHANGED_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_unset_focus_changed_cb(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_error_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_ERROR_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_set_error_cb(muse_camera->camera_handle,
		(camera_error_cb)_camera_dispatcher_error_cb,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_unset_error_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_UNSET_ERROR_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_unset_error_cb(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_foreach_supported_preview_resolution(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_FOREACH_SUPPORTED_PREVIEW_RESOLUTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_foreach_supported_preview_resolution(muse_camera->camera_handle,
		(camera_supported_preview_resolution_cb)_camera_dispatcher_callback_supported_preview_resolution,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_foreach_supported_capture_resolution(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_FOREACH_SUPPORTED_CAPTURE_RESOLUTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_foreach_supported_capture_resolution(muse_camera->camera_handle,
		(camera_supported_capture_resolution_cb)_camera_dispatcher_callback_supported_capture_resolution,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_foreach_supported_capture_format(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_FOREACH_SUPPORTED_CAPTURE_FORMAT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_foreach_supported_capture_format(muse_camera->camera_handle,
		(camera_supported_capture_format_cb)_camera_dispatcher_callback_supported_capture_format,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_foreach_supported_preview_format(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_FOREACH_SUPPORTED_PREVIEW_FORMAT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_foreach_supported_preview_format(muse_camera->camera_handle,
		(camera_supported_preview_format_cb)_camera_dispatcher_callback_supported_preview_format,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_recommended_preview_resolution(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_width = 0;
	int get_height = 0;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_RECOMMENDED_PREVIEW_RESOLUTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_get_recommended_preview_resolution(muse_camera->camera_handle, &get_width, &get_height);
	if (ret == CAMERA_ERROR_NONE) {
		get_value = get_width << 16 | get_height;
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_RECOMMENDED_PREVIEW_RESOLUTION, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_lens_orientation(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_LENS_ORIENTATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_lens_orientation(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_LENS_ORIENTATION, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_theater_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_mode;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_THEATER_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_mode, muse_server_module_get_msg(module));

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_set_theater_mode(muse_camera->camera_handle, (camera_attr_theater_mode_e)set_mode);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_theater_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_theater_mode_e get_value = CAMERA_ATTR_THEATER_MODE_DISABLE;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_THEATER_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_theater_mode(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_THEATER_MODE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_theater_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_THEATER_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);
	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_theater_mode(muse_camera->camera_handle,
		(camera_attr_supported_theater_mode_cb)_camera_dispatcher_callback_supported_theater_mode,
		(void *)module);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_preview_fps(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_fps;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_PREVIEW_FPS;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_fps, muse_server_module_get_msg(module));

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_set_preview_fps(muse_camera->camera_handle, (camera_attr_fps_e)set_fps);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_image_quality(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int quality;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_IMAGE_QUALITY;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(quality, muse_server_module_get_msg(module));

	LOGD("handle : %p, image_quality : %d", muse_camera, quality);

	ret = legacy_camera_attr_set_image_quality(muse_camera->camera_handle, quality);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_preview_fps(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_fps_e get_value = CAMERA_ATTR_FPS_AUTO;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_PREVIEW_FPS;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_preview_fps(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_PREVIEW_FPS, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_image_quality(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_IMAGE_QUALITY;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_image_quality(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_IMAGE_QUALITY, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_zoom(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int zoom;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_ZOOM;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(zoom, muse_server_module_get_msg(module));

	LOGD("handle : %p, zoom : %d", muse_camera, zoom);

	ret = legacy_camera_attr_set_zoom(muse_camera->camera_handle, zoom);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_af_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_mode;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_AF_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_mode, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_mode : %d", muse_camera, set_mode);

	ret = legacy_camera_attr_set_af_mode(muse_camera->camera_handle, (camera_attr_af_mode_e)set_mode);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_af_area(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int value = 0;
	int x = 0;
	int y = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_AF_AREA;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(value, muse_server_module_get_msg(module));

	x = value >> 16;
	y = 0x0000ffff & value;

	LOGD("AF area - %d, %d", x, y);

	ret = legacy_camera_attr_set_af_area(muse_camera->camera_handle, x, y);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_clear_af_area(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_CLEAR_AF_AREA;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_clear_af_area(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_exposure_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_mode;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_EXPOSURE_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_mode, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_mode : %d", muse_camera, set_mode);

	ret = legacy_camera_attr_set_exposure_mode(muse_camera->camera_handle, (camera_attr_exposure_mode_e)set_mode);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_exposure(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int value;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_EXPOSURE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(value, muse_server_module_get_msg(module));

	LOGD("handle : %p, value : %d", muse_camera, value);

	ret = legacy_camera_attr_set_exposure(muse_camera->camera_handle, value);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_iso(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_iso;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_ISO;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_iso, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_iso : %d", muse_camera, set_iso);

	ret = legacy_camera_attr_set_iso(muse_camera->camera_handle, (camera_attr_iso_e)set_iso);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_brightness(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int level;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_BRIGHTNESS;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(level, muse_server_module_get_msg(module));

	LOGD("handle : %p, level : %d", muse_camera, level);

	ret = legacy_camera_attr_set_brightness(muse_camera->camera_handle, level);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_contrast(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int level;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_CONTRAST;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(level, muse_server_module_get_msg(module));

	LOGD("handle : %p, level : %d", muse_camera, level);

	ret = legacy_camera_attr_set_contrast(muse_camera->camera_handle, level);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_hue(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int level;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_HUE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(level, muse_server_module_get_msg(module));

	LOGD("handle : %p, level : %d", muse_camera, level);

	ret = legacy_camera_attr_set_hue(muse_camera->camera_handle, level);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_whitebalance(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_whitebalance;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_WHITEBALANCE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_whitebalance, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_whitebalance : %d", muse_camera, set_whitebalance);

	ret = legacy_camera_attr_set_whitebalance(muse_camera->camera_handle, (camera_attr_whitebalance_e)set_whitebalance);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_effect(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_effect;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_EFFECT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_effect, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_effect : %d", muse_camera, set_effect);

	ret = legacy_camera_attr_set_effect(muse_camera->camera_handle, (camera_attr_effect_mode_e)set_effect);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_scene_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_mode;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_SCENE_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_mode, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_mode : %d", muse_camera, set_mode);

	ret = legacy_camera_attr_set_scene_mode(muse_camera->camera_handle, (camera_attr_scene_mode_e)set_mode);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_enable_tag(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_enable;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_ENABLE_TAG;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_enable, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_enable : %d", muse_camera, set_enable);

	ret = legacy_camera_attr_enable_tag(muse_camera->camera_handle, (bool)set_enable);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_tag_image_description(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_TAG_IMAGE_DESCRIPTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	char description[MUSE_CAMERA_MSG_MAX_LENGTH] = {0,};

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get_string(description, muse_server_module_get_msg(module));

	LOGD("handle : %p, description : %s", muse_camera, description);

	ret = legacy_camera_attr_set_tag_image_description(muse_camera->camera_handle, description);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_tag_orientation(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_orientation;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_TAG_ORIENTATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_orientation, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_orientation : %d", muse_camera, set_orientation);

	ret = legacy_camera_attr_set_tag_orientation(muse_camera->camera_handle, (camera_attr_tag_orientation_e)set_orientation);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_tag_software(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_TAG_SOFTWARE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	char software[MUSE_CAMERA_MSG_MAX_LENGTH] = {0,};

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get_string(software, muse_server_module_get_msg(module));

	LOGD("handle : %p, software : %s", muse_camera, software);

	ret = legacy_camera_attr_set_tag_software(muse_camera->camera_handle, software);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_geotag(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_GEOTAG;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	double set_geotag[3] = {0,};

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get_array(set_geotag, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_geotag1 : %f, set_geotag2 : %f, set_geotag3 : %f",
	     muse_camera, set_geotag[0], set_geotag[1], set_geotag[2]);

	ret = legacy_camera_attr_set_geotag(muse_camera->camera_handle, set_geotag[0], set_geotag[1], set_geotag[2]);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_remove_geotag(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_REMOVE_GEOTAG;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_remove_geotag(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_flash_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	int set_mode = 0;;
	char value_key[KEY_LENGTH] = {'\0',};
	muse_camera_handle_s *muse_camera = NULL;
	camera_device_e device_type = CAMERA_DEVICE_CAMERA0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_FLASH_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_mode, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_mode : %d", muse_camera, set_mode);

	ret = legacy_camera_attr_set_flash_mode(muse_camera->camera_handle, (camera_attr_flash_mode_e)set_mode);
	if (ret == CAMERA_ERROR_NONE) {
		ret = legacy_camera_get_device_type(muse_camera->camera_handle, &device_type);
		if (ret == CAMERA_ERROR_NONE) {
			snprintf(value_key, KEY_LENGTH, "flash_state_camera%d", device_type);
			muse_server_module_set_value(module, value_key, set_mode > 0 ? 1 : 0);
		}
	}

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_zoom(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_ZOOM;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_zoom(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_ZOOM, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_zoom_range(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value0 = 0;
	int get_value1 = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_ZOOM_RANGE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	muse_camera_get_type_e get_type = MUSE_CAMERA_GET_TYPE_INT_PAIR;
	muse_camera_get_int_pair_e index = MUSE_CAMERA_GET_INT_PAIR_ZOOM_RANGE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_zoom_range(muse_camera->camera_handle, &get_value0, &get_value1);
	if (ret == CAMERA_ERROR_NONE)
		muse_camera_msg_return2(api, class, ret, module, get_type, index, get_value0, get_value1);
	else
		muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_af_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_af_mode_e get_value = CAMERA_ATTR_AF_NONE;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_AF_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_af_mode(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_AF_MODE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_exposure_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_exposure_mode_e get_value = CAMERA_ATTR_EXPOSURE_MODE_OFF;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_EXPOSURE_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_exposure_mode(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_EXPOSURE_MODE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_exposure(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_EXPOSURE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_exposure(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_EXPOSURE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_exposure_range(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value0 = 0;
	int get_value1 = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_EXPOSURE_RANGE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	muse_camera_get_type_e get_type = MUSE_CAMERA_GET_TYPE_INT_PAIR;
	muse_camera_get_int_pair_e index = MUSE_CAMERA_GET_INT_PAIR_EXPOSURE_RANGE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_exposure_range(muse_camera->camera_handle, &get_value0, &get_value1);
	if (ret == CAMERA_ERROR_NONE)
		muse_camera_msg_return2(api, class, ret, module, get_type, index, get_value0, get_value1);
	else
		muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_iso(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_iso_e get_value = CAMERA_ATTR_ISO_AUTO;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_ISO;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_iso(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_ISO, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_brightness(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_BRIGHTNESS;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_brightness(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_BRIGHTNESS, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_brightness_range(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value0 = 0;
	int get_value1 = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_BRIGHTNESS_RANGE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	muse_camera_get_type_e get_type = MUSE_CAMERA_GET_TYPE_INT_PAIR;
	muse_camera_get_int_pair_e index = MUSE_CAMERA_GET_INT_PAIR_BRIGHTNESS_RANGE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_brightness_range(muse_camera->camera_handle, &get_value0, &get_value1);
	if (ret == CAMERA_ERROR_NONE)
		muse_camera_msg_return2(api, class, ret, module, get_type, index, get_value0, get_value1);
	else
		muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_contrast(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_CONTRAST;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_contrast(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_CONTRAST, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_contrast_range(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value0 = 0;
	int get_value1 = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_CONTRAST_RANGE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	muse_camera_get_type_e get_type = MUSE_CAMERA_GET_TYPE_INT_PAIR;
	muse_camera_get_int_pair_e index = MUSE_CAMERA_GET_INT_PAIR_CONTRAST_RANGE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_contrast_range(muse_camera->camera_handle, &get_value0, &get_value1);
	if (ret == CAMERA_ERROR_NONE)
		muse_camera_msg_return2(api, class, ret, module, get_type, index, get_value0, get_value1);
	else
		muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_hue(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_HUE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_hue(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_CONTRAST, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_hue_range(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value0 = 0;
	int get_value1 = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_HUE_RANGE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	muse_camera_get_type_e get_type = MUSE_CAMERA_GET_TYPE_INT_PAIR;
	muse_camera_get_int_pair_e index = MUSE_CAMERA_GET_INT_PAIR_HUE_RANGE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_hue_range(muse_camera->camera_handle, &get_value0, &get_value1);
	if (ret == CAMERA_ERROR_NONE)
		muse_camera_msg_return2(api, class, ret, module, get_type, index, get_value0, get_value1);
	else
		muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_whitebalance(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_whitebalance_e get_value = CAMERA_ATTR_WHITE_BALANCE_NONE;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_WHITEBALANCE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_whitebalance(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_WHITE_BALANCE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_effect(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_effect_mode_e get_value = CAMERA_ATTR_EFFECT_NONE;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_EFFECT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_effect(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_EFFECT, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_scene_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_scene_mode_e get_value = CAMERA_ATTR_SCENE_MODE_NORMAL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_SCENE_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_scene_mode(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_SCENE_MODE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_is_enabled_tag(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	bool get_value = false;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_IS_ENABLED_TAG;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_is_enabled_tag(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_ENABLED_TAG, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_tag_image_description(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_TAG_IMAGE_DESCRIPTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	char *get_value = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_tag_image_description(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		LOGD("get_description : %s", get_value);
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_STRING,
			MUSE_CAMERA_GET_STRING_TAG_IMAGE_DESCRIPTION, "get_value", 0, get_value);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	if (get_value) {
		free(get_value);
		get_value = NULL;
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_tag_orientation(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_tag_orientation_e get_value = CAMERA_ATTR_TAG_ORIENTATION_TOP_LEFT;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_TAG_ORIENTATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_tag_orientation(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_TAG_ORIENTATION, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_tag_software(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_TAG_SOFTWARE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	char *get_value = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	ret = legacy_camera_attr_get_tag_software(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		LOGD("get_software : %s", get_value);
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_STRING,
			MUSE_CAMERA_GET_STRING_TAG_SOFTWARE, "get_value", 0, get_value);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	if (get_value) {
		free(get_value);
		get_value = NULL;
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_geotag(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_GEOTAG;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	double get_value[3] = {0.0, 0.0, 0.0};
	char *send_msg = NULL;
	int msg_array_size = 0;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_geotag(muse_camera->camera_handle, &get_value[0], &get_value[1], &get_value[2]);
	if (ret == CAMERA_ERROR_NONE) {
		msg_array_size = sizeof(get_value) / sizeof(int) + (sizeof(get_value) % sizeof(int) ? 1 : 0);

		send_msg = muse_core_msg_new(api,
			MUSE_TYPE_INT, PARAM_API_CLASS, class,
			MUSE_TYPE_INT, PARAM_RET, ret,
			MUSE_TYPE_INT, PARAM_GET_TYPE, MUSE_CAMERA_GET_TYPE_ARRAY,
			MUSE_TYPE_ARRAY, "get_value", msg_array_size, get_value,
			0);

		__camera_dispatcher_send_msg(module, send_msg, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_flash_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	camera_attr_flash_mode_e get_value = CAMERA_ATTR_FLASH_MODE_OFF;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_FLASH_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_flash_mode(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_FLASH_MODE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_af_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_AF_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_af_mode(muse_camera->camera_handle,
		(camera_attr_supported_af_mode_cb)_camera_dispatcher_callback_supported_af_mode,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_exposure_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_EXPOSURE_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p, api : %d", muse_camera, api);

	ret = legacy_camera_attr_foreach_supported_exposure_mode(muse_camera->camera_handle,
		(camera_attr_supported_exposure_mode_cb)_camera_dispatcher_callback_supported_exposure_mode,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_iso(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_ISO;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_iso(muse_camera->camera_handle,
		(camera_attr_supported_iso_cb)_camera_dispatcher_callback_supported_iso_mode,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_whitebalance(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_WHITEBALANCE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_whitebalance(muse_camera->camera_handle,
		(camera_attr_supported_whitebalance_cb)_camera_dispatcher_callback_supported_whitebalance,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_effect(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_EFFECT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_effect(muse_camera->camera_handle,
		(camera_attr_supported_effect_cb)_camera_dispatcher_callback_supported_effect,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_scene_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_SCENE_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_scene_mode(muse_camera->camera_handle,
		(camera_attr_supported_scene_mode_cb)_camera_dispatcher_callback_supported_scene_mode,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_flash_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_FLASH_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_flash_mode(muse_camera->camera_handle,
		(camera_attr_supported_flash_mode_cb)_camera_dispatcher_callback_supported_flash_mode,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_fps(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_FPS;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_fps(muse_camera->camera_handle,
		(camera_attr_supported_fps_cb)_camera_dispatcher_callback_supported_fps,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_fps_by_resolution(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_FPS_BY_RESOLUTION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;
	int value = 0;
	int width = 0;
	int height = 0;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(value, muse_server_module_get_msg(module));

	width = value >> 16;
	height = 0x0000ffff & value;

	LOGD("handle : %p - %dx%d", muse_camera, width, height);

	ret = legacy_camera_attr_foreach_supported_fps_by_resolution(muse_camera->camera_handle,
		width, height,
		(camera_attr_supported_fps_cb)_camera_dispatcher_callback_supported_fps_by_resolution,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_stream_flip(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_STREAM_FLIP;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_stream_flip(muse_camera->camera_handle,
		(camera_attr_supported_stream_flip_cb)_camera_dispatcher_callback_supported_stream_flip,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_stream_rotation(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_STREAM_ROTATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_stream_rotation(muse_camera->camera_handle,
		(camera_attr_supported_stream_rotation_cb)_camera_dispatcher_callback_supported_stream_rotation,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_stream_rotation(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_STREAM_ROTATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	int set_rotation;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_rotation, muse_server_module_get_msg(module));

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_set_stream_rotation(muse_camera->camera_handle, (camera_rotation_e)set_rotation);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_stream_rotation(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_STREAM_ROTATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	camera_rotation_e get_value = CAMERA_ROTATION_NONE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_stream_rotation(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_STREAM_ROTATION, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_stream_flip(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_STREAM_FLIP;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	int set_flip;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_flip, muse_server_module_get_msg(module));

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_set_stream_flip(muse_camera->camera_handle, (camera_flip_e)set_flip);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_stream_flip(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_STREAM_FLIP;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	camera_flip_e get_value = CAMERA_FLIP_NONE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_stream_flip(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_STREAM_FLIP, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_hdr_mode(muse_module_h module)
{
	int set_mode = 0;
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	muse_camera_msg_get(set_mode, muse_server_module_get_msg(module));

	_camera_task_add_job(muse_camera, MUSE_CAMERA_API_ATTR_SET_HDR_MODE,
		MUSE_CAMERA_API_CLASS_IMMEDIATE, set_mode);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_hdr_mode(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_HDR_MODE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	camera_attr_hdr_mode_e get_value = CAMERA_ATTR_HDR_MODE_DISABLE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_hdr_mode(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_HDR_MODE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_is_supported_hdr_capture(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_IS_SUPPORTED_HDR_CAPTURE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_is_supported_hdr_capture(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_hdr_capture_progress_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_HDR_CAPTURE_PROGRESS_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_set_hdr_capture_progress_cb(muse_camera->camera_handle,
		(camera_attr_hdr_progress_cb)_camera_dispatcher_hdr_progress_cb,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_unset_hdr_capture_progress_cb(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_UNSET_HDR_CAPTURE_PROGRESS_CB;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_unset_hdr_capture_progress_cb(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_enable_anti_shake(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_ENABLE_ANTI_SHAKE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	int set_enable;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_enable, muse_server_module_get_msg(module));

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_enable_anti_shake(muse_camera->camera_handle, (bool)set_enable);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_is_enabled_anti_shake(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_IS_ENABLED_ANTI_SHAKE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	bool get_value = false;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_is_enabled_anti_shake(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_ENABLED_ANTI_SHAKE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_is_supported_anti_shake(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_IS_SUPPORTED_ANTI_SHAKE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_is_supported_anti_shake(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_enable_video_stabilization(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_ENABLE_VIDEO_STABILIZATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	int set_enable;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_enable, muse_server_module_get_msg(module));

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_enable_video_stabilization(muse_camera->camera_handle, (bool)set_enable);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_is_enabled_video_stabilization(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_IS_ENABLED_VIDEO_STABILIZATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	bool get_value = false;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_is_enabled_video_stabilization(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_ENABLED_VIDEO_STABILIZATION, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_is_supported_video_stabilization(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_IS_SUPPORTED_VIDEO_STABILIZATION;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_is_supported_video_stabilization(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_enable_auto_contrast(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_ENABLE_AUTO_CONTRAST;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	int set_enable;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_enable, muse_server_module_get_msg(module));

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_enable_auto_contrast(muse_camera->camera_handle, (bool)set_enable);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_is_enabled_auto_contrast(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_IS_ENABLED_AUTO_CONTRAST;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	bool get_value = false;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_is_enabled_auto_contrast(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_ENABLED_AUTO_CONTRAST, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_is_supported_auto_contrast(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_IS_SUPPORTED_AUTO_CONTRAST;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_is_supported_auto_contrast(muse_camera->camera_handle);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_disable_shutter_sound(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_DISABLE_SHUTTER_SOUND;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	int set_disable;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_disable, muse_server_module_get_msg(module));

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_disable_shutter_sound(muse_camera->camera_handle, (bool)set_disable);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_encoded_preview_bitrate(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_ENCODED_PREVIEW_BITRATE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_encoded_preview_bitrate(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_ENCODED_PREVIEW_BITRATE, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_encoded_preview_bitrate(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_bitrate;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_ENCODED_PREVIEW_BITRATE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_bitrate, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_encoded_preview_bitrate : %d", muse_camera, set_bitrate);

	ret = legacy_camera_attr_set_encoded_preview_bitrate(muse_camera->camera_handle, set_bitrate);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_encoded_preview_gop_interval(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_ENCODED_PREVIEW_GOP_INTERVAL;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_encoded_preview_gop_interval(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_ENCODED_PREVIEW_GOP_INTERVAL, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_encoded_preview_gop_interval(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_gop_interval;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_ENCODED_PREVIEW_GOP_INTERVAL;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_gop_interval, muse_server_module_get_msg(module));

	LOGD("handle : %p, set_encoded_preview_gop_interval : %d", muse_camera, set_gop_interval);

	ret = legacy_camera_attr_set_encoded_preview_gop_interval(muse_camera->camera_handle, set_gop_interval);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_pan(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int move_type = 0;
	int pan_step = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_PAN;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(move_type, muse_server_module_get_msg(module));
	muse_camera_msg_get(pan_step, muse_server_module_get_msg(module));

	LOGD("handle : %p - type %d, step %d", muse_camera, move_type, pan_step);

	ret = legacy_camera_attr_set_pan(muse_camera->camera_handle, move_type, pan_step);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_pan(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_PAN;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_pan(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_PAN, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_pan_range(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value0 = 0;
	int get_value1 = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_PAN_RANGE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	muse_camera_get_type_e get_type = MUSE_CAMERA_GET_TYPE_INT_PAIR;
	muse_camera_get_int_pair_e index = MUSE_CAMERA_GET_INT_PAIR_PAN_RANGE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_pan_range(muse_camera->camera_handle, &get_value0, &get_value1);
	if (ret == CAMERA_ERROR_NONE)
		muse_camera_msg_return2(api, class, ret, module, get_type, index, get_value0, get_value1);
	else
		muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_tilt(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int move_type = 0;
	int tilt_step = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_TILT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(move_type, muse_server_module_get_msg(module));
	muse_camera_msg_get(tilt_step, muse_server_module_get_msg(module));

	LOGD("handle : %p - type %d, step %d", muse_camera, move_type, tilt_step);

	ret = legacy_camera_attr_set_tilt(muse_camera->camera_handle, move_type, tilt_step);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_tilt(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_TILT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_tilt(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_TILT, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_tilt_range(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value0 = 0;
	int get_value1 = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_GET_TILT_RANGE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	muse_camera_get_type_e get_type = MUSE_CAMERA_GET_TYPE_INT_PAIR;
	muse_camera_get_int_pair_e index = MUSE_CAMERA_GET_INT_PAIR_TILT_RANGE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_tilt_range(muse_camera->camera_handle, &get_value0, &get_value1);
	if (ret == CAMERA_ERROR_NONE)
		muse_camera_msg_return2(api, class, ret, module, get_type, index, get_value0, get_value1);
	else
		muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_ptz_type(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_ptz_type = 0;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_SET_PTZ_TYPE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_ptz_type, muse_server_module_get_msg(module));

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_set_ptz_type(muse_camera->camera_handle, set_ptz_type);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_foreach_supported_ptz_type(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_PTZ_TYPE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_THREAD_SUB;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_foreach_supported_ptz_type(muse_camera->camera_handle,
		(camera_attr_supported_ptz_type_cb)_camera_dispatcher_callback_supported_ptz_type,
		(void *)module);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_set_display_roi_area(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int set_display_roi_area[4] = {0,};
	muse_camera_api_e api = MUSE_CAMERA_API_SET_DISPLAY_ROI_AREA;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get_array(set_display_roi_area, muse_server_module_get_msg(module));

	LOGD("handle : %p, x : %d, y : %d, width : %d, height : %d", muse_camera,
		set_display_roi_area[0], set_display_roi_area[1], set_display_roi_area[2], set_display_roi_area[3]);

	ret = legacy_camera_attr_set_display_roi_area(muse_camera->camera_handle, set_display_roi_area);

	LOGD("ret : 0x%x", ret);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_attr_get_display_roi_area(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	muse_camera_handle_s *muse_camera = NULL;
	int get_value[4] = {0,};
	muse_camera_api_e api = MUSE_CAMERA_API_GET_DISPLAY_ROI_AREA;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;
	char *send_msg = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	LOGD("handle : %p", muse_camera);

	ret = legacy_camera_attr_get_display_roi_area(muse_camera->camera_handle, get_value);
	if (ret == CAMERA_ERROR_NONE) {
		send_msg = muse_core_msg_new(api,
			MUSE_TYPE_INT, PARAM_API_CLASS, class,
			MUSE_TYPE_INT, PARAM_RET, ret,
			MUSE_TYPE_INT, PARAM_GET_TYPE, MUSE_CAMERA_GET_TYPE_ARRAY,
			MUSE_TYPE_ARRAY, "get_value", 4, get_value,
			0);

		__camera_dispatcher_send_msg(module, send_msg, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}


int camera_dispatcher_return_buffer(muse_module_h module)
{
	int ret_fd = 0;
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(ret_fd, muse_server_module_get_msg(module));

	/*LOGD("ret_fd : %d", ret_fd);*/

	if (!_camera_remove_export_data(module, (tbm_fd)ret_fd, FALSE))
		LOGE("remove export data failed : fd %d", ret_fd);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_preview_cb_return(muse_module_h module)
{
	muse_camera_handle_s *muse_camera = NULL;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	if (muse_camera == NULL) {
		LOGE("NULL handle");
		return MUSE_CAMERA_ERROR_NONE;
	}

	/*LOGD("ENTER");*/

	g_mutex_lock(&muse_camera->preview_cb_lock);
	g_cond_signal(&muse_camera->preview_cb_cond);
	/*LOGD("send signal for preview callback");*/
	g_mutex_unlock(&muse_camera->preview_cb_lock);

	/*LOGD("DONE");*/

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_set_display_reuse_hint(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	int set_hint = 0;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_SET_DISPLAY_REUSE_HINT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	muse_camera_msg_get(set_hint, muse_server_module_get_msg(module));

	LOGD("set hint : %d", set_hint);

	ret = legacy_camera_set_display_reuse_hint(muse_camera->camera_handle, (bool)set_hint);

	muse_camera_msg_return(api, class, ret, module);

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_display_reuse_hint(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	int get_value = 0;
	muse_camera_handle_s *muse_camera = NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_DISPLAY_REUSE_HINT;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);

	ret = legacy_camera_get_display_reuse_hint(muse_camera->camera_handle, &get_value);
	if (ret == CAMERA_ERROR_NONE) {
		LOGD("hint : %d", get_value);
		muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
			MUSE_CAMERA_GET_INT_DISPLAY_REUSE_HINT, "get_value", get_value, NULL);
	} else {
		muse_camera_msg_return(api, class, ret, module);
	}

	return MUSE_CAMERA_ERROR_NONE;
}

int camera_dispatcher_get_device_state(muse_module_h module)
{
	int ret = CAMERA_ERROR_NONE;
	int is_called = false;
	char value_key[KEY_LENGTH] = {'\0',};
	bool camera_feature_supported = false;
	camera_h camera = NULL;
	camera_device_e device_type = CAMERA_DEVICE_CAMERA0;
	camera_device_state_e get_device_state = CAMERA_DEVICE_STATE_NULL;
	muse_camera_api_e api = MUSE_CAMERA_API_GET_DEVICE_STATE;
	muse_camera_api_class_e class = MUSE_CAMERA_API_CLASS_IMMEDIATE;

	muse_camera_msg_get(device_type, muse_server_module_get_msg(module));

	if (device_type > CAMERA_DEVICE_CAMERA9) {
		LOGE("invalid device %d", device_type);

		ret = CAMERA_ERROR_INVALID_PARAMETER;
		muse_camera_msg_return(api, class, ret, module);

		return MUSE_CAMERA_ERROR_NONE;
	}

	muse_server_module_get_value(module, MUSED_KEY_DEVICE_STATE_CHECK, &is_called);

	if (!is_called) {
		ret = legacy_camera_create(device_type, &camera);
		if (camera) {
			legacy_camera_destroy(camera);
			camera = NULL;
		}

		/* camera feature support check */
		if (ret == CAMERA_ERROR_NONE) {
			if (muse_server_get_platform_info(CAMERA_FEATURE_NAME, &camera_feature_supported) != MM_ERROR_NONE)
				LOGE("failed to get platform info for camera support");

			if (!camera_feature_supported) {
				LOGE("CAMERA NOT SUPPORTED, BUT INI FILE FOR CAMERA IS EXISTED. RECOMMEND TO REMOVE IT.");
				ret = CAMERA_ERROR_NOT_SUPPORTED;
			}
		}

		muse_server_module_set_value(module, MUSED_KEY_DEVICE_STATE_RETURN, ret);
		muse_server_module_set_value(module, MUSED_KEY_DEVICE_STATE_CHECK, (int)true);
	} else {
		muse_server_module_get_value(module, MUSED_KEY_DEVICE_STATE_RETURN, &ret);
	}

	LOGD("is_called %d, ret 0x%x", is_called, ret);

	if (ret != CAMERA_ERROR_NONE) {
		LOGE("failed to create camera handle 0x%x", ret);

		muse_camera_msg_return(api, class, ret, module);

		return MUSE_CAMERA_ERROR_NONE;
	}

	snprintf(value_key, KEY_LENGTH, "device_state_camera%d", device_type);
	muse_server_module_get_value(module, value_key, (int *)&get_device_state);

	LOGD("device[%d] state : %d", device_type, get_device_state);

	muse_camera_msg_return1(api, class, ret, module, MUSE_CAMERA_GET_TYPE_INT,
		-1, "get_device_state", get_device_state, NULL);

	return MUSE_CAMERA_ERROR_NONE;
}

int (*dispatcher[MUSE_CAMERA_API_MAX]) (muse_module_h module) = {
	camera_dispatcher_create, /* MUSE_CAMERA_API_CREATE */
	camera_dispatcher_destroy, /* MUSE_CAMERA_API_DESTROY */
	camera_dispatcher_start_preview, /* MUSE_CAMERA_START_PREVIEW */
	camera_dispatcher_stop_preview, /* MUSE_CAMERA_API_START_PREVIEW */
	camera_dispatcher_start_capture, /* MUSE_CAMERA_START_CAPTURE */
	camera_dispatcher_is_supported_continuous_capture, /* MUSE_CAMERA_API_SUPPORT_CONTINUOUS_CAPTURE */
	camera_dispatcher_start_continuous_capture, /* MUSE_CAMERA_API_START_CONTINUOUS_CAPTURE, */
	camera_dispatcher_stop_continuous_capture, /* MUSE_CAMERA_API_STOP_CONTINUOUS_CAPTURE, */
	camera_dispatcher_is_supported_face_detection, /* MUSE_CAMERA_API_SUPPORT_FACE_DETECTION, */
	camera_dispatcher_is_supported_zero_shutter_lag, /* MUSE_CAMERA_API_SUPPORT_ZERO_SHUTTER_LAG, */
	camera_dispatcher_is_supported_media_packet_preview_cb, /* MUSE_CAMERA_API_SUPPORT_MEDIA_PACKET_PREVIEW_CB, */
	camera_dispatcher_get_device_count, /* MUSE_CAMERA_API_GET_DEVICE_COUNT, */
	camera_dispatcher_start_face_detection, /* MUSE_CAMERA_API_START_FACE_DETECTION, */
	camera_dispatcher_stop_face_detection, /* MUSE_CAMERA_API_STOP_FACE_DETECTION, */
	camera_dispatcher_get_state, /* MUSE_CAMERA_API_GET_STATE, */
	camera_dispatcher_start_focusing, /* MUSE_CAMERA_API_START_FOCUSING, */
	camera_dispatcher_stop_focusing, /* MUSE_CAMERA_API_CANCEL_FOCUSING, */
	camera_dispatcher_set_display, /* MUSE_CAMERA_API_SET_DISPLAY, */
	camera_dispatcher_set_preview_resolution, /* MUSE_CAMERA_API_SET_PREVIEW_RESOLUTION, */
	camera_dispatcher_set_capture_resolution, /* MUSE_CAMERA_API_SET_CAPTURE_RESOLUTION, */
	camera_dispatcher_set_capture_format, /* MUSE_CAMERA_API_SET_CAPTURE_FORMAT, */
	camera_dispatcher_set_preview_format, /* MUSE_CAMERA_API_SET_PREVIEW_FORMAT, */
	camera_dispatcher_get_preview_resolution, /* MUSE_CAMERA_API_GET_PREVIEW_RESOLUTION, */
	camera_dispatcher_set_display_rotation, /* MUSE_CAMERA_API_SET_DISPLAY_ROTATION, */
	camera_dispatcher_get_display_rotation, /* MUSE_CAMERA_API_GET_DISPLAY_ROTATION, */
	camera_dispatcher_set_display_flip, /* MUSE_CAMERA_API_SET_DISPLAY_FLIP, */
	camera_dispatcher_get_display_flip, /* MUSE_CAMERA_API_GET_DISPLAY_FLIP, */
	camera_dispatcher_set_display_visible, /* MUSE_CAMERA_API_SET_DISPLAY_VISIBLE, */
	camera_dispatcher_is_display_visible, /* MUSE_CAMERA_API_IS_DISPLAY_VISIBLE, */
	camera_dispatcher_set_display_mode, /* MUSE_CAMERA_API_SET_DISPLAY_MODE, */
	camera_dispatcher_get_display_mode, /* MUSE_CAMERA_API_GET_DISPLAY_MODE, */
	camera_dispatcher_get_capture_resolution, /* MUSE_CAMERA_API_GET_CAPTURE_RESOLUTION, */
	camera_dispatcher_get_capture_format, /* MUSE_CAMERA_API_GET_CAPTURE_FORMAT, */
	camera_dispatcher_get_preview_format, /* MUSE_CAMERA_API_GET_PREVIEW_FORMAT, */
	camera_dispatcher_get_facing_direction, /* MUSE_CAMERA_API_GET_FACING_DIRECTION, */
	camera_dispatcher_get_flash_state, /* MUSE_CAMERA_API_GET_FLASH_STATE, */
	camera_dispatcher_set_preview_cb, /* MUSE_CAMERA_API_SET_PREVIEW_CB, */
	camera_dispatcher_unset_preview_cb, /* MUSE_CAMERA_API_UNSET_PREVIEW_CB, */
	camera_dispatcher_set_media_packet_preview_cb, /* MUSE_CAMERA_API_SET_MEDIA_PACKET_PREVIEW_CB, */
	camera_dispatcher_unset_media_packet_preview_cb, /* MUSE_CAMERA_API_UNSET_MEDIA_PACKET_PREVIEW_CB, */
	camera_dispatcher_set_state_changed_cb, /* MUSE_CAMERA_API_SET_STATE_CHANGED_CB, */
	camera_dispatcher_unset_state_changed_cb, /* MUSE_CAMERA_API_UNSET_STATE_CHANGED_CB, */
	camera_dispatcher_set_interrupted_cb, /* MUSE_CAMERA_API_SET_INTERRUPTED_CB, */
	camera_dispatcher_unset_interrupted_cb, /* MUSE_CAMERA_API_UNSET_INTERRUPTED_CB, */
	camera_dispatcher_set_focus_changed_cb, /* MUSE_CAMERA_API_SET_FOCUS_CHANGED_CB, */
	camera_dispatcher_unset_focus_changed_cb, /* MUSE_CAMERA_API_UNSET_FOCUS_CHANGED_CB, */
	camera_dispatcher_set_error_cb, /* MUSE_CAMERA_API_SET_ERROR_CB, */
	camera_dispatcher_unset_error_cb, /* MUSE_CAMERA_API_UNSET_ERROR_CB, */
	camera_dispatcher_foreach_supported_preview_resolution, /* MUSE_CAMERA_API_SET_FOREACH_SUPPORTED_PREVIEW_RESOLUTION, */
	camera_dispatcher_foreach_supported_capture_resolution, /* MUSE_CAMERA_API_SET_FOREACH_SUPPORTED_CAPTURE_RESOLUTION, */
	camera_dispatcher_foreach_supported_capture_format, /* MUSE_CAMERA_API_SET_FOREACH_SUPPORTED_CAPTURE_FORMAT, */
	camera_dispatcher_foreach_supported_preview_format, /* MUSE_CAMERA_API_SET_FOREACH_SUPPORTED_PREVIEW_FORMAT, */
	camera_dispatcher_get_recommended_preview_resolution, /* MUSE_CAMERA_API_GET_RECOMMENDED_PREVIEW_RESOLUTION, */
	camera_dispatcher_attr_get_lens_orientation, /* MUSE_CAMERA_API_ATTR_GET_LENS_ORIENTATION, */
	camera_dispatcher_attr_set_theater_mode, /* MUSE_CAMERA_API_ATTR_SET_THEATER_MODE, */
	camera_dispatcher_attr_get_theater_mode, /* MUSE_CAMERA_API_ATTR_GET_THEATER_MODE, */
	camera_dispatcher_attr_foreach_supported_theater_mode, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_THEATER_MODE, */
	camera_dispatcher_attr_set_preview_fps, /* MUSE_CAMERA_API_ATTR_SET_PREVIEW_FPS, */
	camera_dispatcher_attr_set_image_quality, /* MUSE_CAMERA_API_ATTR_SET_IMAGE_QUALITY, */
	camera_dispatcher_attr_get_preview_fps, /* MUSE_CAMERA_API_ATTR_GET_PREVIEW_FPS, */
	camera_dispatcher_attr_get_image_quality, /* MUSE_CAMERA_API_ATTR_GET_IMAGE_QUALITY, */
	camera_dispatcher_attr_set_zoom, /* MUSE_CAMERA_API_ATTR_SET_ZOOM, */
	camera_dispatcher_attr_set_af_mode, /* MUSE_CAMERA_API_ATTR_SET_AF_MODE, */
	camera_dispatcher_attr_set_af_area, /* MUSE_CAMERA_API_ATTR_SET_AF_AREA, */
	camera_dispatcher_attr_clear_af_area, /* MUSE_CAMERA_API_ATTR_CLEAR_AF_AREA, */
	camera_dispatcher_attr_set_exposure_mode, /* MUSE_CAMERA_API_ATTR_SET_EXPOSURE_MODE, */
	camera_dispatcher_attr_set_exposure, /* MUSE_CAMERA_API_ATTR_SET_EXPOSURE, */
	camera_dispatcher_attr_set_iso, /* MUSE_CAMERA_API_ATTR_SET_ISO, */
	camera_dispatcher_attr_set_brightness, /* MUSE_CAMERA_API_ATTR_SET_BRIGHTNESS, */
	camera_dispatcher_attr_set_contrast, /* MUSE_CAMERA_API_ATTR_SET_CONTRAST, */
	camera_dispatcher_attr_set_whitebalance, /* MUSE_CAMERA_API_ATTR_SET_WHITEBALANCE, */
	camera_dispatcher_attr_set_effect, /* MUSE_CAMERA_API_ATTR_SET_EFFECT, */
	camera_dispatcher_attr_set_scene_mode, /* MUSE_CAMERA_API_ATTR_SET_SCENE_MODE, */
	camera_dispatcher_attr_enable_tag, /* MUSE_CAMERA_API_ATTR_ENABLE_TAG, */
	camera_dispatcher_attr_set_tag_image_description, /* MUSE_CAMERA_API_ATTR_SET_TAG_IMAGE_DESCRIPTION, */
	camera_dispatcher_attr_set_tag_orientation, /* MUSE_CAMERA_API_ATTR_SET_TAG_ORIENTATION, */
	camera_dispatcher_attr_set_tag_software, /* MUSE_CAMERA_API_ATTR_SET_TAG_SOFTWARE, */
	camera_dispatcher_attr_set_geotag, /* MUSE_CAMERA_API_ATTR_SET_GEOTAG, */
	camera_dispatcher_attr_remove_geotag, /* MUSE_CAMERA_API_ATTR_REMOVE_GEOTAG, */
	camera_dispatcher_attr_set_flash_mode, /* MUSE_CAMERA_API_ATTR_SET_FLASH_MODE, */
	camera_dispatcher_attr_get_zoom, /* MUSE_CAMERA_API_ATTR_GET_ZOOM, */
	camera_dispatcher_attr_get_zoom_range, /* MUSE_CAMERA_API_ATTR_GET_ZOOM_RANGE, */
	camera_dispatcher_attr_get_af_mode, /* MUSE_CAMERA_API_ATTR_GET_AF_MODE, */
	camera_dispatcher_attr_get_exposure_mode, /* MUSE_CAMERA_API_ATTR_GET_EXPOSURE_MODE, */
	camera_dispatcher_attr_get_exposure, /* MUSE_CAMERA_API_ATTR_GET_EXPOSURE, */
	camera_dispatcher_attr_get_exposure_range, /* MUSE_CAMERA_API_ATTR_GET_EXPOSURE_RANGE, */
	camera_dispatcher_attr_get_iso, /* MUSE_CAMERA_API_ATTR_GET_ISO, */
	camera_dispatcher_attr_get_brightness, /* MUSE_CAMERA_API_ATTR_GET_BRIGHTNESS, */
	camera_dispatcher_attr_get_brightness_range, /* MUSE_CAMERA_API_ATTR_GET_BRIGHTNESS_RANGE, */
	camera_dispatcher_attr_get_contrast, /* MUSE_CAMERA_API_ATTR_GET_CONTRAST, */
	camera_dispatcher_attr_get_contrast_range, /* MUSE_CAMERA_API_ATTR_GET_CONTRAST_RANGE, */
	camera_dispatcher_attr_get_whitebalance, /* MUSE_CAMERA_API_ATTR_GET_WHITEBALANCE, */
	camera_dispatcher_attr_get_effect, /* MUSE_CAMERA_API_ATTR_GET_EFFECT, */
	camera_dispatcher_attr_get_scene_mode, /* MUSE_CAMERA_API_ATTR_GET_SCENE_MODE, */
	camera_dispatcher_attr_is_enabled_tag, /* MUSE_CAMERA_API_ATTR_IS_ENABLED_TAG, */
	camera_dispatcher_attr_get_tag_image_description, /* MUSE_CAMERA_API_ATTR_GET_TAG_IMAGE_DESCRIPTION, */
	camera_dispatcher_attr_get_tag_orientation, /* MUSE_CAMERA_API_ATTR_GET_TAG_ORIENTATION, */
	camera_dispatcher_attr_get_tag_software, /* MUSE_CAMERA_API_ATTR_GET_TAG_SOFTWARE, */
	camera_dispatcher_attr_get_geotag, /* MUSE_CAMERA_API_ATTR_GET_GEOTAG, */
	camera_dispatcher_attr_get_flash_mode, /* MUSE_CAMERA_API_ATTR_GET_FLASH_MODE, */
	camera_dispatcher_attr_foreach_supported_af_mode, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_AF_MODE, */
	camera_dispatcher_attr_foreach_supported_exposure_mode, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_EXPOSURE_MODE, */
	camera_dispatcher_attr_foreach_supported_iso, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_ISO, */
	camera_dispatcher_attr_foreach_supported_whitebalance, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_WHITEBALANCE, */
	camera_dispatcher_attr_foreach_supported_effect, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_EFFECT, */
	camera_dispatcher_attr_foreach_supported_scene_mode, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_SCENE_MODE, */
	camera_dispatcher_attr_foreach_supported_flash_mode, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_FLASH_MODE, */
	camera_dispatcher_attr_foreach_supported_fps, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_FPS, */
	camera_dispatcher_attr_foreach_supported_fps_by_resolution, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_FPS_BY_RESOLUTION, */
	camera_dispatcher_attr_foreach_supported_stream_flip, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_STREAM_FLIP, */
	camera_dispatcher_attr_foreach_supported_stream_rotation, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_STREAM_ROTATION, */
	camera_dispatcher_attr_set_stream_rotation, /* MUSE_CAMERA_API_ATTR_SET_STREAM_ROTATION, */
	camera_dispatcher_attr_get_stream_rotation, /* MUSE_CAMERA_API_ATTR_GET_STREAM_ROTATION, */
	camera_dispatcher_attr_set_stream_flip, /* MUSE_CAMERA_API_ATTR_SET_STREAM_FLIP, */
	camera_dispatcher_attr_get_stream_flip, /* MUSE_CAMERA_API_ATTR_GET_STREAM_FLIP, */
	camera_dispatcher_attr_set_hdr_mode, /* MUSE_CAMERA_API_ATTR_SET_HDR_MODE, */
	camera_dispatcher_attr_get_hdr_mode, /* MUSE_CAMERA_API_ATTR_GET_HDR_MODE, */
	camera_dispatcher_attr_is_supported_hdr_capture, /* MUSE_CAMERA_API_ATTR_IS_SUPPORTED_HDR_CAPTURE, */
	camera_dispatcher_attr_set_hdr_capture_progress_cb, /* MUSE_CAMERA_API_ATTR_SET_HDR_CAPTURE_PROGRESS_CB, */
	camera_dispatcher_attr_unset_hdr_capture_progress_cb, /* MUSE_CAMERA_API_ATTR_UNSET_HDR_CAPTURE_PROGRESS_CB, */
	camera_dispatcher_attr_enable_anti_shake, /* MUSE_CAMERA_API_ATTR_ENABLE_ANTI_SHAKE, */
	camera_dispatcher_attr_is_enabled_anti_shake, /* MUSE_CAMERA_API_ATTR_IS_ENABLED_ANTI_SHAKE, */
	camera_dispatcher_attr_is_supported_anti_shake, /* MUSE_CAMERA_API_ATTR_IS_SUPPORTED_ANTI_SHAKE, */
	camera_dispatcher_attr_enable_video_stabilization, /* MUSE_CAMERA_API_ATTR_ENABLE_VIDEO_STABILIZATION, */
	camera_dispatcher_attr_is_enabled_video_stabilization, /* MUSE_CAMERA_API_ATTR_IS_ENABLED_VIDEO_STABILIZATION, */
	camera_dispatcher_attr_is_supported_video_stabilization, /* MUSE_CAMERA_API_ATTR_IS_SUPPORTED_VIDEO_STABILIZATION, */
	camera_dispatcher_attr_enable_auto_contrast, /* MUSE_CAMERA_API_ATTR_ENABLE_AUTO_CONTRAST, */
	camera_dispatcher_attr_is_enabled_auto_contrast, /* MUSE_CAMERA_API_ATTR_IS_ENABLED_AUTO_CONTRAST, */
	camera_dispatcher_attr_is_supported_auto_contrast, /* MUSE_CAMERA_API_ATTR_IS_SUPPORTED_AUTO_CONTRAST, */
	camera_dispatcher_attr_disable_shutter_sound, /* MUSE_CAMERA_API_ATTR_DISABLE_SHUTTER_SOUND, */
	camera_dispatcher_attr_get_encoded_preview_bitrate, /* MUSE_CAMERA_API_ATTR_GET_ENCODED_PREVIEW_BITRATE, */
	camera_dispatcher_attr_set_encoded_preview_bitrate, /* MUSE_CAMERA_API_ATTR_SET_ENCODED_PREVIEW_BITRATE, */
	camera_dispatcher_attr_get_encoded_preview_gop_interval, /* MUSE_CAMERA_API_ATTR_GET_ENCODED_PREVIEW_GOP_INTERVAL, */
	camera_dispatcher_attr_set_encoded_preview_gop_interval, /* MUSE_CAMERA_API_ATTR_SET_ENCODED_PREVIEW_GOP_INTERVAL, */
	camera_dispatcher_attr_set_pan, /* MUSE_CAMERA_API_ATTR_SET_PAN */
	camera_dispatcher_attr_get_pan, /* MUSE_CAMERA_API_ATTR_GET_PAN */
	camera_dispatcher_attr_get_pan_range, /* MUSE_CAMERA_API_ATTR_GET_PAN_RANGE */
	camera_dispatcher_attr_set_tilt, /* MUSE_CAMERA_API_ATTR_SET_TILT */
	camera_dispatcher_attr_get_tilt, /* MUSE_CAMERA_API_ATTR_GET_TILT */
	camera_dispatcher_attr_get_tilt_range, /* MUSE_CAMERA_API_ATTR_GET_TILT_RANGE */
	camera_dispatcher_attr_set_ptz_type, /* MUSE_CAMERA_API_ATTR_SET_PTZ_TYPE */
	camera_dispatcher_attr_foreach_supported_ptz_type, /* MUSE_CAMERA_API_ATTR_FOREACH_SUPPORTED_PTZ_TYPE */
	camera_dispatcher_attr_set_display_roi_area, /* MUSE_CAMERA_API_SET_DISPLAY_ROI_AREA */
	camera_dispatcher_attr_get_display_roi_area, /* MUSE_CAMERA_API_GET_DISPLAY_ROI_AREA */
	camera_dispatcher_return_buffer, /* MUSE_CAMERA_API_RETURN_BUFFER */
	camera_dispatcher_preview_cb_return, /* MUSE_CAMERA_API_PREVIEW_CB_RETURN */
	camera_dispatcher_set_display_reuse_hint, /* MUSE_CAMERA_API_SET_DISPLAY_REUSE_HINT */
	camera_dispatcher_get_display_reuse_hint, /* MUSE_CAMERA_API_GET_DISPLAY_REUSE_HINT */
	camera_dispatcher_change_device, /* MUSE_CAMERA_API_CHANGE_DEVICE */
	camera_dispatcher_get_device_state, /* MUSE_CAMERA_API_GET_DEVICE_STATE */
	camera_dispatcher_set_interrupt_started_cb, /* MUSE_CAMERA_API_SET_INTERRUPT_STARTED_CB, */
	camera_dispatcher_unset_interrupt_started_cb, /* MUSE_CAMERA_API_UNSET_INTERRUPT_STARTED_CB, */
	camera_dispatcher_attr_set_hue, /* MUSE_CAMERA_API_ATTR_SET_HUE, */
	camera_dispatcher_attr_get_hue, /* MUSE_CAMERA_API_ATTR_GET_HUE, */
	camera_dispatcher_attr_get_hue_range /* MUSE_CAMERA_API_ATTR_GET_HUE_RANGE, */
};


/******************/
/* cmd dispatcher */
/******************/
static int camera_cmd_dispatcher_initialize(muse_module_h module)
{
	int item_count = 0;
	int i = 0;
	GstPlugin *plugin = NULL;

	const char *load_list[] = {
		LIBDIR"/gstreamer-1.0/libgstcoreelements.so",
		LIBDIR"/gstreamer-1.0/libgstcamerasrc.so",
		LIBDIR"/gstreamer-1.0/libgstwaylandsink.so",
	};

	item_count = sizeof(load_list) / sizeof(load_list[0]);

	LOGD("item count %d", item_count);

	for (i = 0 ; i < item_count ; i++) {
		plugin = gst_plugin_load_file(load_list[i], NULL);
		if (plugin) {
			LOGD("%s loaded", load_list[i]);
			gst_object_unref(plugin);
			plugin = NULL;
		} else {
			LOGW("failed to load %s", load_list[i]);
		}
	}

	LOGD("done");

	return MUSE_CAMERA_ERROR_NONE;
}


static int camera_cmd_dispatcher_shutdown(muse_module_h module)
{
	muse_camera_handle_s *muse_camera = NULL;
	camera_state_e state = CAMERA_STATE_NONE;
	int capture_try_count = 0;
	int ret = 0;

	muse_camera = (muse_camera_handle_s *)muse_server_ipc_get_handle(module);
	if (muse_camera == NULL) {
		LOGE("NULL handle");
		return MUSE_CAMERA_ERROR_INVALID;
	}

	legacy_camera_lock(muse_camera->camera_handle, true);

	if (legacy_camera_is_used(muse_camera->camera_handle)) {
		LOGW("camera is used in recorder.. wait...");
		if (legacy_camera_wait(muse_camera->camera_handle, 3000)) {
			LOGD("recorder is released");
		} else {
			legacy_camera_lock(muse_camera->camera_handle, false);
			LOGE("wait timeout, could not release camera handle %p", muse_camera->camera_handle);
			return MUSE_CAMERA_ERROR_INVALID;
		}
	}

	legacy_camera_lock(muse_camera->camera_handle, false);

	muse_camera->is_shutting_down = true;

again:
	legacy_camera_get_state(muse_camera->camera_handle, &state);

	LOGW("current state : %d", state);

	switch (state) {
	case CAMERA_STATE_CAPTURING:
		if (capture_try_count < 30) {
			LOGW("now capturing.. wait for capture data...");
			usleep(100 * 1000);
			capture_try_count++;
			goto again;
		} else {
			LOGE("wait capture data timeout! keep going...");
		}
		/* fall through */
	case CAMERA_STATE_CAPTURED:
		legacy_camera_start_preview(muse_camera->camera_handle);
		/* fall through */
	case CAMERA_STATE_PREVIEW:
		_camera_remove_export_data(module, -1, TRUE); /* force return buffer before stop preview */
		legacy_camera_stop_preview(muse_camera->camera_handle);
		/* fall through */
	case CAMERA_STATE_CREATED:
		ret = legacy_camera_destroy(muse_camera->camera_handle);
		if (ret == CAMERA_ERROR_NONE)
			_camera_dispatcher_release_resource(module);
		else
			LOGE("failed to destroy camera handle");

		break;
	default:
		break;
	}

	LOGW("done");

	return MUSE_CAMERA_ERROR_NONE;
}


int (*cmd_dispatcher[MUSE_MODULE_COMMAND_MAX])(muse_module_h module) = {
	camera_cmd_dispatcher_initialize, /* MUSE_MODULE_COMMAND_INITIALIZE */
	camera_cmd_dispatcher_shutdown, /* MUSE_MODULE_COMMAND_SHUTDOWN */
	NULL, /* MUSE_MODULE_COMMAND_DEBUG_INFO_DUMP */
	NULL, /* MUSE_MODULE_COMMAND_CREATE_SERVER_ACK */
	NULL, /* MUSE_MODULE_COMMAND_RESOURCE_NOT_AVAILABLE */
	NULL  /* MUSE_MODULE_COMMAND_EXTERNAL_STORAGE_STATE_CHANGED */
};
